#include <sys/types.h>
#include <signal.h>

#include <stdio.h>
#include "kdump_subr.h"

/*
 * These are simple support macros. print_or utilizes a variable
 * defined in the calling function to track whether or not it should
 * print a logical-OR character ('|') before a string. if_print_or
 * simply handles the necessary "if" statement used in many lines
 * of this file.
 */
#define print_or(str,orflag) do {			\
	if (orflag != 0)				\
		putchar('|');				\
	printf (str);					\
	} while (0)

#define if_print_or(i,flag,flag2,orflag) do {		\
	if ((i == flag) || ((i & flag) == flag)) {	\
		print_or(#flag2,orflag);		\
		orflag |= flag;				\
	}						\
} while (0)

const struct linux_error_msg linux_errors[]= {
	{NULL, NULL}
,	{"EPERM", "Operation not permitted"}
,	{"ENOENT", "No such file or directory"}
,	{"ESRCH", "No such process"}
,	{"EINTR", "Interrupted system call"}
,	{"EIO", "I/O error"}
,	{"ENXIO", "No such device or address"}
,	{"E2BIG", "Argument list too long"}
,	{"ENOEXEC", "Exec format error"}
,	{"EBADF", "Bad file number"}
,	{"ECHILD", "No child processes"}
,	{"EAGAIN", "Try again"}
,	{"ENOMEM", "Out of memory"}
,	{"EACCES", "Permission denied"}
,	{"EFAULT", "Bad address"}
,	{"ENOTBLK", "Block device required"}
,	{"EBUSY", "Device or resource busy"}
,	{"EEXIST", "File exists"}
,	{"EXDEV", "Cross-device link"}
,	{"ENODEV", "No such device"}
,	{"ENOTDIR", "Not a directory"}
,	{"EISDIR", "Is a directory"}
,	{"EINVAL", "Invalid argument"}
,	{"ENFILE", "File table overflow"}
,	{"EMFILE", "Too many open files"}
,	{"ENOTTY", "Not a typewriter"}
,	{"ETXTBSY", "Text file busy"}
,	{"EFBIG", "File too large"}
,	{"ENOSPC", "No space left on device"}
,	{"ESPIPE", "Illegal seek"}
,	{"EROFS", "Read-only file system"}
,	{"EMLINK", "Too many links"}
,	{"EPIPE", "Broken pipe"}
,	{"EDOM", "Math argument out of domain of func"}
,	{"ERANGE", "Math result not representable"}
,	{"EDEADLK", "Resource deadlock would occur"}
,	{"ENAMETOOLONG", "File name too long"}
,	{"ENOLCK", "No record locks available"}
,	{"ENOSYS", "Function not implemented"}
,	{"ENOTEMPTY", "Directory not empty"}
,	{"ELOOP", "Too many symbolic links encountered"}
,	{NULL, NULL}
,	{"ENOMSG", "No message of desired type"}
,	{"EIDRM", "Identifier removed"}
,	{"ECHRNG", "Channel number out of range"}
,	{"EL2NSYNC", "Level 2 not synchronized"}
,	{"EL3HLT", "Level 3 halted"}
,	{"EL3RST", "Level 3 reset"}
,	{"ELNRNG", "Link number out of range"}
,	{"EUNATCH", "Protocol driver not attached"}
,	{"ENOCSI", "No CSI structure available"}
,	{"EL2HLT", "Level 2 halted"}
,	{"EBADE", "Invalid exchange"}
,	{"EBADR", "Invalid request descriptor"}
,	{"EXFULL", "Exchange full"}
,	{"ENOANO", "No anode"}
,	{"EBADRQC", "Invalid request code"}
,	{"EBADSLT", "Invalid slot"}
,	{NULL, NULL}
,	{"EBFONT", "Bad font file format"}
,	{"ENOSTR", "Device not a stream"}
,	{"ENODATA", "No data available"}
,	{"ETIME", "Timer expired"}
,	{"ENOSR", "Out of streams resources"}
,	{"ENONET", "Machine is not on the network"}
,	{"ENOPKG", "Package not installed"}
,	{"EREMOTE", "Object is remote"}
,	{"ENOLINK", "Link has been severed"}
,	{"EADV", "Advertise error"}
,	{"ESRMNT", "Srmount error"}
,	{"ECOMM", "Communication error on send"}
,	{"EPROTO", "Protocol error"}
,	{"EMULTIHOP", "Multihop attempted"}
,	{"EDOTDOT", "RFS specific error"}
,	{"EBADMSG", "Not a data message"}
,	{"EOVERFLOW", "Value too large for defined data type"}
,	{"ENOTUNIQ", "Name not unique on network"}
,	{"EBADFD", "File descriptor in bad state"}
,	{"EREMCHG", "Remote address changed"}
,	{"ELIBACC", "Can not access a needed shared library"}
,	{"ELIBBAD", "Accessing a corrupted shared library"}
,	{"ELIBSCN", ".lib section in a.out corrupted"}
,	{"ELIBMAX", "Attempting to link in too many shared libraries"}
,	{"ELIBEXEC", "Cannot exec a shared library directly"}
,	{"EILSEQ", "Illegal byte sequence"}
,	{"ERESTART", "Interrupted system call should be restarted"}
,	{"ESTRPIPE", "Streams pipe error"}
,	{"EUSERS", "Too many users"}
,	{"ENOTSOCK", "Socket operation on non-socket"}
,	{"EDESTADDRREQ", "Destination address required"}
,	{"EMSGSIZE", "Message too long"}
,	{"EPROTOTYPE", "Protocol wrong type for socket"}
,	{"ENOPROTOOPT", "Protocol not available"}
,	{"EPROTONOSUPPORT", "Protocol not supported"}
,	{"ESOCKTNOSUPPORT", "Socket type not supported"}
,	{"EOPNOTSUPP", "Operation not supported on transport endpoint"}
,	{"EPFNOSUPPORT", "Protocol family not supported"}
,	{"EAFNOSUPPORT", "Address family not supported by protocol"}
,	{"EADDRINUSE", "Address already in use"}
,	{"EADDRNOTAVAIL", "Cannot assign requested address"}
,	{"ENETDOWN", "Network is down"}
,	{"ENETUNREACH", "Network is unreachable"}
,	{"ENETRESET", "Network dropped connection because of reset"}
,	{"ECONNABORTED", "Software caused connection abort"}
,	{"ECONNRESET", "Connection reset by peer"}
,	{"ENOBUFS", "No buffer space available"}
,	{"EISCONN", "Transport endpoint is already connected"}
,	{"ENOTCONN", "Transport endpoint is not connected"}
,	{"ESHUTDOWN", "Cannot send after transport endpoint shutdown"}
,	{"ETOOMANYREFS", "Too many references: cannot splice"}
,	{"ETIMEDOUT", "Connection timed out"}
,	{"ECONNREFUSED", "Connection refused"}
,	{"EHOSTDOWN", "Host is down"}
,	{"EHOSTUNREACH", "No route to host"}
,	{"EALREADY", "Operation already in progress"}
,	{"EINPROGRESS", "Operation now in progress"}
,	{"ESTALE", "Stale NFS file handle"}
,	{"EUCLEAN", "Structure needs cleaning"}
,	{"ENOTNAM", "Not a XENIX named type file"}
,	{"ENAVAIL", "No XENIX semaphores available"}
,	{"EISNAM", "Is a named type file"}
,	{"EREMOTEIO", "Remote I/O error"}
,	{"EDQUOT", "Quota exceeded"}
,	{"ENOMEDIUM", "No medium found"}
,	{"EMEDIUMTYPE", "Wrong medium type"}
,	{"ECANCELED", "Operation Canceled"}
,	{"ENOKEY", "Required key not available"}
,	{"EKEYEXPIRED", "Key has expired"}
,	{"EKEYREVOKED", "Key has been revoked"}
,	{"EKEYREJECTED", "Key was rejected by service"}
,	{"EOWNERDEAD", "Owner died"}
,	{"ENOTRECOVERABLE", "State not recoverable"}
};

int nlinux_errors = sizeof(linux_errors) / sizeof(linux_errors[0]);

const char* linux_signames[]= {
	NULL, "HUP", "INT", "QUIT", "ILL", "TRAP", 
	"ABRT", "BUS", "FPE", "KILL", "USR1", "SEGV", 
	"USR2", "PIPE", "ALRM", "TERM", "STKFLT", "CHLD", 
	"CONT", "STOP", "TSTP", "TTIN", "TTOU", "URG", 
	"XCPU", "XFSZ", "VTALRM", "PROF", "WINCH", "IO", 
	"PWR", "SYS", NULL };

int nlinux_signames = (sizeof(linux_signames) / sizeof(linux_signames[0]))-1;

/*
 * MANUAL
 *
 */
void
linux_signame(int sig)
{

	if (sig > 0 && sig < nlinux_signames)
		printf("SIG%s", linux_signames[sig]);
	else if (sig == 0)
		printf("%d", sig);
	else
		printf("SIG %d", sig);
}


/*
 * auto-generated from /usr/local/gentoo-stage3/usr/include/asm/mman.h
 */
#define LINUX_MCL_CURRENT		1
#define LINUX_MCL_FUTURE		2



/* AUTO */
void
linux_mlockallname(int arg)
{
	int	or = 0;
	if (arg==LINUX_MCL_CURRENT || !((LINUX_MCL_CURRENT & or)==LINUX_MCL_CURRENT))
		if_print_or(arg, LINUX_MCL_CURRENT, MCL_CURRENT, or);
	if (arg==LINUX_MCL_FUTURE || !((LINUX_MCL_FUTURE & or)==LINUX_MCL_FUTURE))
		if_print_or(arg, LINUX_MCL_FUTURE, MCL_FUTURE, or);
	if (or == 0 && or != arg)
		printf("<invalid>%ld", ((long)arg)^or);
}


/*
 * auto-generated from /usr/local/gentoo-stage3/usr/include/linux/sched.h
 */
#define LINUX_CLONE_VM		0x00000100
#define LINUX_CLONE_FS		0x00000200
#define LINUX_CLONE_FILES		0x00000400
#define LINUX_CLONE_SIGHAND		0x00000800
#define LINUX_CLONE_PTRACE		0x00002000
#define LINUX_CLONE_VFORK		0x00004000
#define LINUX_CLONE_PARENT		0x00008000
#define LINUX_CLONE_THREAD		0x00010000
#define LINUX_CLONE_NEWNS		0x00020000
#define LINUX_CLONE_SYSVSEM		0x00040000
#define LINUX_CLONE_SETTLS		0x00080000
#define LINUX_CLONE_PARENT_SETTID		0x00100000
#define LINUX_CLONE_CHILD_CLEARTID		0x00200000
#define LINUX_CLONE_DETACHED		0x00400000
#define LINUX_CLONE_UNTRACED		0x00800000
#define LINUX_CLONE_CHILD_SETTID		0x01000000
#define LINUX_CLONE_STOPPED		0x02000000
#define LINUX_CLONE_NEWUTS		0x04000000
#define LINUX_CLONE_NEWIPC		0x08000000
#define LINUX_CLONE_NEWUSER		0x10000000



/* AUTO */
void
linux_cloneflagsname(int arg)
{
	int	or = 0;
	if (arg==LINUX_CLONE_VM || !((LINUX_CLONE_VM & or)==LINUX_CLONE_VM))
		if_print_or(arg, LINUX_CLONE_VM, CLONE_VM, or);
	if (arg==LINUX_CLONE_FS || !((LINUX_CLONE_FS & or)==LINUX_CLONE_FS))
		if_print_or(arg, LINUX_CLONE_FS, CLONE_FS, or);
	if (arg==LINUX_CLONE_FILES || !((LINUX_CLONE_FILES & or)==LINUX_CLONE_FILES))
		if_print_or(arg, LINUX_CLONE_FILES, CLONE_FILES, or);
	if (arg==LINUX_CLONE_SIGHAND || !((LINUX_CLONE_SIGHAND & or)==LINUX_CLONE_SIGHAND))
		if_print_or(arg, LINUX_CLONE_SIGHAND, CLONE_SIGHAND, or);
	if (arg==LINUX_CLONE_PTRACE || !((LINUX_CLONE_PTRACE & or)==LINUX_CLONE_PTRACE))
		if_print_or(arg, LINUX_CLONE_PTRACE, CLONE_PTRACE, or);
	if (arg==LINUX_CLONE_VFORK || !((LINUX_CLONE_VFORK & or)==LINUX_CLONE_VFORK))
		if_print_or(arg, LINUX_CLONE_VFORK, CLONE_VFORK, or);
	if (arg==LINUX_CLONE_PARENT || !((LINUX_CLONE_PARENT & or)==LINUX_CLONE_PARENT))
		if_print_or(arg, LINUX_CLONE_PARENT, CLONE_PARENT, or);
	if (arg==LINUX_CLONE_THREAD || !((LINUX_CLONE_THREAD & or)==LINUX_CLONE_THREAD))
		if_print_or(arg, LINUX_CLONE_THREAD, CLONE_THREAD, or);
	if (arg==LINUX_CLONE_NEWNS || !((LINUX_CLONE_NEWNS & or)==LINUX_CLONE_NEWNS))
		if_print_or(arg, LINUX_CLONE_NEWNS, CLONE_NEWNS, or);
	if (arg==LINUX_CLONE_SYSVSEM || !((LINUX_CLONE_SYSVSEM & or)==LINUX_CLONE_SYSVSEM))
		if_print_or(arg, LINUX_CLONE_SYSVSEM, CLONE_SYSVSEM, or);
	if (arg==LINUX_CLONE_SETTLS || !((LINUX_CLONE_SETTLS & or)==LINUX_CLONE_SETTLS))
		if_print_or(arg, LINUX_CLONE_SETTLS, CLONE_SETTLS, or);
	if (arg==LINUX_CLONE_PARENT_SETTID || !((LINUX_CLONE_PARENT_SETTID & or)==LINUX_CLONE_PARENT_SETTID))
		if_print_or(arg, LINUX_CLONE_PARENT_SETTID, CLONE_PARENT_SETTID, or);
	if (arg==LINUX_CLONE_CHILD_CLEARTID || !((LINUX_CLONE_CHILD_CLEARTID & or)==LINUX_CLONE_CHILD_CLEARTID))
		if_print_or(arg, LINUX_CLONE_CHILD_CLEARTID, CLONE_CHILD_CLEARTID, or);
	if (arg==LINUX_CLONE_DETACHED || !((LINUX_CLONE_DETACHED & or)==LINUX_CLONE_DETACHED))
		if_print_or(arg, LINUX_CLONE_DETACHED, CLONE_DETACHED, or);
	if (arg==LINUX_CLONE_UNTRACED || !((LINUX_CLONE_UNTRACED & or)==LINUX_CLONE_UNTRACED))
		if_print_or(arg, LINUX_CLONE_UNTRACED, CLONE_UNTRACED, or);
	if (arg==LINUX_CLONE_CHILD_SETTID || !((LINUX_CLONE_CHILD_SETTID & or)==LINUX_CLONE_CHILD_SETTID))
		if_print_or(arg, LINUX_CLONE_CHILD_SETTID, CLONE_CHILD_SETTID, or);
	if (arg==LINUX_CLONE_STOPPED || !((LINUX_CLONE_STOPPED & or)==LINUX_CLONE_STOPPED))
		if_print_or(arg, LINUX_CLONE_STOPPED, CLONE_STOPPED, or);
	if (arg==LINUX_CLONE_NEWUTS || !((LINUX_CLONE_NEWUTS & or)==LINUX_CLONE_NEWUTS))
		if_print_or(arg, LINUX_CLONE_NEWUTS, CLONE_NEWUTS, or);
	if (arg==LINUX_CLONE_NEWIPC || !((LINUX_CLONE_NEWIPC & or)==LINUX_CLONE_NEWIPC))
		if_print_or(arg, LINUX_CLONE_NEWIPC, CLONE_NEWIPC, or);
	if (arg==LINUX_CLONE_NEWUSER || !((LINUX_CLONE_NEWUSER & or)==LINUX_CLONE_NEWUSER))
		if_print_or(arg, LINUX_CLONE_NEWUSER, CLONE_NEWUSER, or);
	if (or == 0 && or != arg)
		printf("<invalid>%ld", ((long)arg)^or);
}


/*
 * auto-generated from /usr/local/gentoo-stage3/usr/include/linux/mman.h
 */
#define LINUX_MREMAP_MAYMOVE		1
#define LINUX_MREMAP_FIXED		2



/* AUTO */
void
linux_mremapflagsname(int arg)
{
	int	or = 0;
	if (arg==LINUX_MREMAP_MAYMOVE || !((LINUX_MREMAP_MAYMOVE & or)==LINUX_MREMAP_MAYMOVE))
		if_print_or(arg, LINUX_MREMAP_MAYMOVE, MREMAP_MAYMOVE, or);
	if (arg==LINUX_MREMAP_FIXED || !((LINUX_MREMAP_FIXED & or)==LINUX_MREMAP_FIXED))
		if_print_or(arg, LINUX_MREMAP_FIXED, MREMAP_FIXED, or);
	if (or == 0 && or != arg)
		printf("<invalid>%ld", ((long)arg)^or);
}


/*
 * auto-generated from /usr/local/gentoo-stage3/usr/include/linux/fs.h
 */
#define LINUX_MS_RDONLY		1
#define LINUX_MS_NOSUID		2
#define LINUX_MS_NODEV		4
#define LINUX_MS_NOEXEC		8
#define LINUX_MS_SYNCHRONOUS		16
#define LINUX_MS_REMOUNT		32
#define LINUX_MS_MANDLOCK		64
#define LINUX_MS_DIRSYNC		128
#define LINUX_MS_NOATIME		1024
#define LINUX_MS_NODIRATIME		2048
#define LINUX_MS_BIND		4096
#define LINUX_MS_MOVE		8192
#define LINUX_MS_REC		16384
#define LINUX_MS_VERBOSE		32768



/* AUTO */
void
linux_mountflagsname(int arg)
{
	int	or = 0;
	if (arg==LINUX_MS_RDONLY || !((LINUX_MS_RDONLY & or)==LINUX_MS_RDONLY))
		if_print_or(arg, LINUX_MS_RDONLY, MS_RDONLY, or);
	if (arg==LINUX_MS_NOSUID || !((LINUX_MS_NOSUID & or)==LINUX_MS_NOSUID))
		if_print_or(arg, LINUX_MS_NOSUID, MS_NOSUID, or);
	if (arg==LINUX_MS_NODEV || !((LINUX_MS_NODEV & or)==LINUX_MS_NODEV))
		if_print_or(arg, LINUX_MS_NODEV, MS_NODEV, or);
	if (arg==LINUX_MS_NOEXEC || !((LINUX_MS_NOEXEC & or)==LINUX_MS_NOEXEC))
		if_print_or(arg, LINUX_MS_NOEXEC, MS_NOEXEC, or);
	if (arg==LINUX_MS_SYNCHRONOUS || !((LINUX_MS_SYNCHRONOUS & or)==LINUX_MS_SYNCHRONOUS))
		if_print_or(arg, LINUX_MS_SYNCHRONOUS, MS_SYNCHRONOUS, or);
	if (arg==LINUX_MS_REMOUNT || !((LINUX_MS_REMOUNT & or)==LINUX_MS_REMOUNT))
		if_print_or(arg, LINUX_MS_REMOUNT, MS_REMOUNT, or);
	if (arg==LINUX_MS_MANDLOCK || !((LINUX_MS_MANDLOCK & or)==LINUX_MS_MANDLOCK))
		if_print_or(arg, LINUX_MS_MANDLOCK, MS_MANDLOCK, or);
	if (arg==LINUX_MS_DIRSYNC || !((LINUX_MS_DIRSYNC & or)==LINUX_MS_DIRSYNC))
		if_print_or(arg, LINUX_MS_DIRSYNC, MS_DIRSYNC, or);
	if (arg==LINUX_MS_NOATIME || !((LINUX_MS_NOATIME & or)==LINUX_MS_NOATIME))
		if_print_or(arg, LINUX_MS_NOATIME, MS_NOATIME, or);
	if (arg==LINUX_MS_NODIRATIME || !((LINUX_MS_NODIRATIME & or)==LINUX_MS_NODIRATIME))
		if_print_or(arg, LINUX_MS_NODIRATIME, MS_NODIRATIME, or);
	if (arg==LINUX_MS_BIND || !((LINUX_MS_BIND & or)==LINUX_MS_BIND))
		if_print_or(arg, LINUX_MS_BIND, MS_BIND, or);
	if (arg==LINUX_MS_MOVE || !((LINUX_MS_MOVE & or)==LINUX_MS_MOVE))
		if_print_or(arg, LINUX_MS_MOVE, MS_MOVE, or);
	if (arg==LINUX_MS_REC || !((LINUX_MS_REC & or)==LINUX_MS_REC))
		if_print_or(arg, LINUX_MS_REC, MS_REC, or);
	if (arg==LINUX_MS_VERBOSE || !((LINUX_MS_VERBOSE & or)==LINUX_MS_VERBOSE))
		if_print_or(arg, LINUX_MS_VERBOSE, MS_VERBOSE, or);
	if (or == 0 && or != arg)
		printf("<invalid>%ld", ((long)arg)^or);
}


/*
 * auto-generated from /usr/local/gentoo-stage3/usr/include/asm/signal.h
 */
#define LINUX_SA_NOCLDSTOP		0x00000001u
#define LINUX_SA_NOCLDWAIT		0x00000002u
#define LINUX_SA_SIGINFO		0x00000004u
#define LINUX_SA_ONSTACK		0x08000000u
#define LINUX_SA_RESTART		0x10000000u
#define LINUX_SA_NODEFER		0x40000000u
#define LINUX_SA_RESETHAND		0x80000000u
#define LINUX_SA_RESTORER		0x04000000



/* AUTO */
void
linux_sigactionflagsname(int arg)
{
	int	or = 0;
	if (arg==LINUX_SA_NOCLDSTOP || !((LINUX_SA_NOCLDSTOP & or)==LINUX_SA_NOCLDSTOP))
		if_print_or(arg, LINUX_SA_NOCLDSTOP, SA_NOCLDSTOP, or);
	if (arg==LINUX_SA_NOCLDWAIT || !((LINUX_SA_NOCLDWAIT & or)==LINUX_SA_NOCLDWAIT))
		if_print_or(arg, LINUX_SA_NOCLDWAIT, SA_NOCLDWAIT, or);
	if (arg==LINUX_SA_SIGINFO || !((LINUX_SA_SIGINFO & or)==LINUX_SA_SIGINFO))
		if_print_or(arg, LINUX_SA_SIGINFO, SA_SIGINFO, or);
	if (arg==LINUX_SA_ONSTACK || !((LINUX_SA_ONSTACK & or)==LINUX_SA_ONSTACK))
		if_print_or(arg, LINUX_SA_ONSTACK, SA_ONSTACK, or);
	if (arg==LINUX_SA_RESTART || !((LINUX_SA_RESTART & or)==LINUX_SA_RESTART))
		if_print_or(arg, LINUX_SA_RESTART, SA_RESTART, or);
	if (arg==LINUX_SA_NODEFER || !((LINUX_SA_NODEFER & or)==LINUX_SA_NODEFER))
		if_print_or(arg, LINUX_SA_NODEFER, SA_NODEFER, or);
	if (arg==LINUX_SA_RESETHAND || !((LINUX_SA_RESETHAND & or)==LINUX_SA_RESETHAND))
		if_print_or(arg, LINUX_SA_RESETHAND, SA_RESETHAND, or);
	if (arg==LINUX_SA_RESTORER || !((LINUX_SA_RESTORER & or)==LINUX_SA_RESTORER))
		if_print_or(arg, LINUX_SA_RESTORER, SA_RESTORER, or);
	if (or == 0 && or != arg)
		printf("<invalid>%ld", ((long)arg)^or);
}


/* AUTO */
void
linux_prctloptionname(int arg)
{
	switch (arg) {
	case 1:
		printf("PR_SET_PDEATHSIG");
		break;
	case 2:
		printf("PR_GET_PDEATHSIG");
		break;
	case 3:
		printf("PR_GET_DUMPABLE");
		break;
	case 4:
		printf("PR_SET_DUMPABLE");
		break;
	case 5:
		printf("PR_GET_UNALIGN");
		break;
	case 6:
		printf("PR_SET_UNALIGN");
		break;
	case 7:
		printf("PR_GET_KEEPCAPS");
		break;
	case 8:
		printf("PR_SET_KEEPCAPS");
		break;
	case 9:
		printf("PR_GET_FPEMU");
		break;
	case 10:
		printf("PR_SET_FPEMU");
		break;
	case 11:
		printf("PR_GET_FPEXC");
		break;
	case 12:
		printf("PR_SET_FPEXC");
		break;
	case 13:
		printf("PR_GET_TIMING");
		break;
	case 14:
		printf("PR_SET_TIMING");
		break;
	case 15:
		printf("PR_SET_NAME");
		break;
	case 16:
		printf("PR_GET_NAME");
		break;
	case 19:
		printf("PR_GET_ENDIAN");
		break;
	case 20:
		printf("PR_SET_ENDIAN");
		break;
	case 21:
		printf("PR_GET_SECCOMP");
		break;
	case 22:
		printf("PR_SET_SECCOMP");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}


/* AUTO */
void
linux_sockoptname(int arg)
{
	switch (arg) {
	case 1:
		printf("SO_DEBUG");
		break;
	case 2:
		printf("SO_REUSEADDR");
		break;
	case 3:
		printf("SO_TYPE");
		break;
	case 4:
		printf("SO_ERROR");
		break;
	case 5:
		printf("SO_DONTROUTE");
		break;
	case 6:
		printf("SO_BROADCAST");
		break;
	case 7:
		printf("SO_SNDBUF");
		break;
	case 8:
		printf("SO_RCVBUF");
		break;
	case 32:
		printf("SO_SNDBUFFORCE");
		break;
	case 33:
		printf("SO_RCVBUFFORCE");
		break;
	case 9:
		printf("SO_KEEPALIVE");
		break;
	case 10:
		printf("SO_OOBINLINE");
		break;
	case 12:
		printf("SO_PRIORITY");
		break;
	case 13:
		printf("SO_LINGER");
		break;
	case 14:
		printf("SO_BSDCOMPAT");
		break;
	case 16:
		printf("SO_PASSCRED");
		break;
	case 17:
		printf("SO_PEERCRED");
		break;
	case 18:
		printf("SO_RCVLOWAT");
		break;
	case 19:
		printf("SO_SNDLOWAT");
		break;
	case 20:
		printf("SO_RCVTIMEO");
		break;
	case 21:
		printf("SO_SNDTIMEO");
		break;
	case 25:
		printf("SO_BINDTODEVICE");
		break;
	case 28:
		printf("SO_PEERNAME");
		break;
	case 29:
		printf("SO_TIMESTAMP");
		break;
	case 30:
		printf("SO_ACCEPTCONN");
		break;
	case 31:
		printf("SO_PEERSEC");
		break;
	case 34:
		printf("SO_PASSSEC");
		break;
	case 35:
		printf("SO_TIMESTAMPNS");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}


/* AUTO */
void
linux_rebootoptname(int arg)
{
	switch (arg) {
	case 0x01234567:
		printf("LINUX_REBOOT_CMD_RESTART");
		break;
	case 0xCDEF0123:
		printf("LINUX_REBOOT_CMD_HALT");
		break;
	case 0x89ABCDEF:
		printf("LINUX_REBOOT_CMD_CAD_ON");
		break;
	case 0x00000000:
		printf("LINUX_REBOOT_CMD_CAD_OFF");
		break;
	case 0x4321FEDC:
		printf("LINUX_REBOOT_CMD_POWER_OFF");
		break;
	case 0xA1B2C3D4:
		printf("LINUX_REBOOT_CMD_RESTART2");
		break;
	case 0xD000FCE2:
		printf("LINUX_REBOOT_CMD_SW_SUSPEND");
		break;
	case 0x45584543:
		printf("LINUX_REBOOT_CMD_KEXEC");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}


/* AUTO */
void
linux_rlimitname(int arg)
{
	switch (arg) {
	case 0:
		printf("RLIMIT_CPU");
		break;
	case 1:
		printf("RLIMIT_FSIZE");
		break;
	case 2:
		printf("RLIMIT_DATA");
		break;
	case 3:
		printf("RLIMIT_STACK");
		break;
	case 4:
		printf("RLIMIT_CORE");
		break;
	case 5:
		printf("RLIMIT_RSS");
		break;
	case 6:
		printf("RLIMIT_NPROC");
		break;
	case 7:
		printf("RLIMIT_NOFILE");
		break;
	case 8:
		printf("RLIMIT_MEMLOCK");
		break;
	case 9:
		printf("RLIMIT_AS");
		break;
	case 10:
		printf("RLIMIT_LOCKS");
		break;
	case 11:
		printf("RLIMIT_SIGPENDING");
		break;
	case 12:
		printf("RLIMIT_MSGQUEUE");
		break;
	case 13:
		printf("RLIMIT_NICE");
		break;
	case 14:
		printf("RLIMIT_RTPRIO");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}


/* AUTO */
void
linux_prioname(int arg)
{
	switch (arg) {
	case (-20):
		printf("PRIO_MIN");
		break;
	case 20:
		printf("PRIO_MAX");
		break;
	case 0:
		printf("PRIO_PROCESS");
		break;
	case 1:
		printf("PRIO_PGRP");
		break;
	case 2:
		printf("PRIO_USER");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}


/* AUTO */
void
linux_rusagename(int arg)
{
	switch (arg) {
	case 0:
		printf("RUSAGE_SELF");
		break;
	case (-1):
		printf("RUSAGE_CHILDREN");
		break;
	case (-2):
		printf("RUSAGE_BOTH");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}


/* AUTO */
void
linux_itimername(int arg)
{
	switch (arg) {
	case 0:
		printf("ITIMER_REAL");
		break;
	case 1:
		printf("ITIMER_VIRTUAL");
		break;
	case 2:
		printf("ITIMER_PROF");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}


/* AUTO */
void
linux_schedpolicyname(int arg)
{
	switch (arg) {
	case 0:
		printf("SCHED_NORMAL");
		break;
	case 1:
		printf("SCHED_FIFO");
		break;
	case 2:
		printf("SCHED_RR");
		break;
	case 3:
		printf("SCHED_BATCH");
		break;
	case 5:
		printf("SCHED_IDLE");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}


/* AUTO */
void
linux_fadvisename(int arg)
{
	switch (arg) {
	case 0:
		printf("POSIX_FADV_NORMAL");
		break;
	case 1:
		printf("POSIX_FADV_RANDOM");
		break;
	case 2:
		printf("POSIX_FADV_SEQUENTIAL");
		break;
	case 3:
		printf("POSIX_FADV_WILLNEED");
		break;
	case 6:
		printf("POSIX_FADV_DONTNEED");
		break;
	case 7:
		printf("POSIX_FADV_NOREUSE");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}


/* AUTO */
void
linux_clocktidname(int arg)
{
	switch (arg) {
	case 0:
		printf("CLOCK_REALTIME");
		break;
	case 1:
		printf("CLOCK_MONOTONIC");
		break;
	case 2:
		printf("CLOCK_PROCESS_CPUTIME_ID");
		break;
	case 3:
		printf("CLOCK_THREAD_CPUTIME_ID");
		break;
	case 10:
		printf("CLOCK_SGI_CYCLE");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}


/* AUTO */
void
linux_clockflagsname(int arg)
{
	switch (arg) {
	case 0x01:
		printf("TIMER_ABSTIME");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}


/* AUTO */
void
linux_mempolicyname(int arg)
{
	switch (arg) {
	case 0:
		printf("MPOL_DEFAULT");
		break;
	case 1:
		printf("MPOL_PREFERRED");
		break;
	case 2:
		printf("MPOL_BIND");
		break;
	case 3:
		printf("MPOL_INTERLEAVE");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}

/*
 * auto-generated from /usr/local/gentoo-stage3/usr/include/linux/in.h
 */
#define LINUX_IPPROTO_IP		0
#define LINUX_IPPROTO_ICMP		1
#define LINUX_IPPROTO_IGMP		2
#define LINUX_IPPROTO_IPIP		4
#define LINUX_IPPROTO_TCP		6
#define LINUX_IPPROTO_EGP		8
#define LINUX_IPPROTO_PUP		12
#define LINUX_IPPROTO_UDP		17
#define LINUX_IPPROTO_IDP		22
#define LINUX_IPPROTO_DCCP		33
#define LINUX_IPPROTO_RSVP		46
#define LINUX_IPPROTO_GRE		47
#define LINUX_IPPROTO_IPV6		41
#define LINUX_IPPROTO_ESP		50
#define LINUX_IPPROTO_AH		51
#define LINUX_IPPROTO_BEETPH		94
#define LINUX_IPPROTO_PIM		103
#define LINUX_IPPROTO_COMP		108
#define LINUX_IPPROTO_SCTP		132
#define LINUX_IPPROTO_UDPLITE		136
#define LINUX_IPPROTO_RAW		255
#define LINUX_IPPROTO_MAX		256



/* AUTO */
void
linux_sockipprotoname(int arg)
{
	if (arg == LINUX_IPPROTO_IP) 
		printf("IPPROTO_IP");
	else if (arg == LINUX_IPPROTO_ICMP) 
		printf("IPPROTO_ICMP");
	else if (arg == LINUX_IPPROTO_IGMP) 
		printf("IPPROTO_IGMP");
	else if (arg == LINUX_IPPROTO_IPIP) 
		printf("IPPROTO_IPIP");
	else if (arg == LINUX_IPPROTO_TCP) 
		printf("IPPROTO_TCP");
	else if (arg == LINUX_IPPROTO_EGP) 
		printf("IPPROTO_EGP");
	else if (arg == LINUX_IPPROTO_PUP) 
		printf("IPPROTO_PUP");
	else if (arg == LINUX_IPPROTO_UDP) 
		printf("IPPROTO_UDP");
	else if (arg == LINUX_IPPROTO_IDP) 
		printf("IPPROTO_IDP");
	else if (arg == LINUX_IPPROTO_DCCP) 
		printf("IPPROTO_DCCP");
	else if (arg == LINUX_IPPROTO_RSVP) 
		printf("IPPROTO_RSVP");
	else if (arg == LINUX_IPPROTO_GRE) 
		printf("IPPROTO_GRE");
	else if (arg == LINUX_IPPROTO_IPV6) 
		printf("IPPROTO_IPV6");
	else if (arg == LINUX_IPPROTO_ESP) 
		printf("IPPROTO_ESP");
	else if (arg == LINUX_IPPROTO_AH) 
		printf("IPPROTO_AH");
	else if (arg == LINUX_IPPROTO_BEETPH) 
		printf("IPPROTO_BEETPH");
	else if (arg == LINUX_IPPROTO_PIM) 
		printf("IPPROTO_PIM");
	else if (arg == LINUX_IPPROTO_COMP) 
		printf("IPPROTO_COMP");
	else if (arg == LINUX_IPPROTO_SCTP) 
		printf("IPPROTO_SCTP");
	else if (arg == LINUX_IPPROTO_UDPLITE) 
		printf("IPPROTO_UDPLITE");
	else if (arg == LINUX_IPPROTO_RAW) 
		printf("IPPROTO_RAW");
	else if (arg == LINUX_IPPROTO_MAX) 
		printf("IPPROTO_MAX");
	else /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
}


/*
 * auto-generated from /usr/local/gentoo-stage3/usr/include/asm-generic/fcntl.h
 */
#define LINUX_O_ACCMODE		00000003
#define LINUX_O_RDONLY		00000000
#define LINUX_O_WRONLY		00000001
#define LINUX_O_RDWR		00000002
#define LINUX_O_CREAT		00000100
#define LINUX_O_EXCL		00000200
#define LINUX_O_NOCTTY		00000400
#define LINUX_O_TRUNC		00001000
#define LINUX_O_APPEND		00002000
#define LINUX_O_NONBLOCK		00004000
#define LINUX_O_SYNC		00010000
#define LINUX_O_DIRECT		00040000
#define LINUX_O_LARGEFILE		00100000
#define LINUX_O_DIRECTORY		00200000
#define LINUX_O_NOFOLLOW		00400000
#define LINUX_O_NOATIME		01000000
#define LINUX_O_CLOEXEC		02000000



/* AUTO */
void
linux_flagsname(int arg)
{
	int	or = 0;
	if (arg==LINUX_O_ACCMODE || !((LINUX_O_ACCMODE & or)==LINUX_O_ACCMODE))
		if_print_or(arg, LINUX_O_ACCMODE, O_ACCMODE, or);
	if (arg==LINUX_O_RDONLY || !((LINUX_O_RDONLY & or)==LINUX_O_RDONLY))
		if_print_or(arg, LINUX_O_RDONLY, O_RDONLY, or);
	if (arg==LINUX_O_WRONLY || !((LINUX_O_WRONLY & or)==LINUX_O_WRONLY))
		if_print_or(arg, LINUX_O_WRONLY, O_WRONLY, or);
	if (arg==LINUX_O_RDWR || !((LINUX_O_RDWR & or)==LINUX_O_RDWR))
		if_print_or(arg, LINUX_O_RDWR, O_RDWR, or);
	if (arg==LINUX_O_CREAT || !((LINUX_O_CREAT & or)==LINUX_O_CREAT))
		if_print_or(arg, LINUX_O_CREAT, O_CREAT, or);
	if (arg==LINUX_O_EXCL || !((LINUX_O_EXCL & or)==LINUX_O_EXCL))
		if_print_or(arg, LINUX_O_EXCL, O_EXCL, or);
	if (arg==LINUX_O_NOCTTY || !((LINUX_O_NOCTTY & or)==LINUX_O_NOCTTY))
		if_print_or(arg, LINUX_O_NOCTTY, O_NOCTTY, or);
	if (arg==LINUX_O_TRUNC || !((LINUX_O_TRUNC & or)==LINUX_O_TRUNC))
		if_print_or(arg, LINUX_O_TRUNC, O_TRUNC, or);
	if (arg==LINUX_O_APPEND || !((LINUX_O_APPEND & or)==LINUX_O_APPEND))
		if_print_or(arg, LINUX_O_APPEND, O_APPEND, or);
	if (arg==LINUX_O_NONBLOCK || !((LINUX_O_NONBLOCK & or)==LINUX_O_NONBLOCK))
		if_print_or(arg, LINUX_O_NONBLOCK, O_NONBLOCK, or);
	if (arg==LINUX_O_SYNC || !((LINUX_O_SYNC & or)==LINUX_O_SYNC))
		if_print_or(arg, LINUX_O_SYNC, O_SYNC, or);
	if (arg==LINUX_O_DIRECT || !((LINUX_O_DIRECT & or)==LINUX_O_DIRECT))
		if_print_or(arg, LINUX_O_DIRECT, O_DIRECT, or);
	if (arg==LINUX_O_LARGEFILE || !((LINUX_O_LARGEFILE & or)==LINUX_O_LARGEFILE))
		if_print_or(arg, LINUX_O_LARGEFILE, O_LARGEFILE, or);
	if (arg==LINUX_O_DIRECTORY || !((LINUX_O_DIRECTORY & or)==LINUX_O_DIRECTORY))
		if_print_or(arg, LINUX_O_DIRECTORY, O_DIRECTORY, or);
	if (arg==LINUX_O_NOFOLLOW || !((LINUX_O_NOFOLLOW & or)==LINUX_O_NOFOLLOW))
		if_print_or(arg, LINUX_O_NOFOLLOW, O_NOFOLLOW, or);
	if (arg==LINUX_O_NOATIME || !((LINUX_O_NOATIME & or)==LINUX_O_NOATIME))
		if_print_or(arg, LINUX_O_NOATIME, O_NOATIME, or);
	if (arg==LINUX_O_CLOEXEC || !((LINUX_O_CLOEXEC & or)==LINUX_O_CLOEXEC))
		if_print_or(arg, LINUX_O_CLOEXEC, O_CLOEXEC, or);
	if (or == 0 && or != arg)
		printf("<invalid>%ld", ((long)arg)^or);
}


/*
 * auto-generated from /usr/local/gentoo-stage3/usr/include/asm-generic/fcntl.h
 */
#define LINUX_LOCK_SH		1
#define LINUX_LOCK_EX		2
#define LINUX_LOCK_NB		4
#define LINUX_LOCK_UN		8
#define LINUX_LOCK_MAND		32
#define LINUX_LOCK_READ		64
#define LINUX_LOCK_WRITE		128
#define LINUX_LOCK_RW		192



/* AUTO */
void
linux_flockname(int arg)
{
	int	or = 0;
	if (arg==LINUX_LOCK_SH || !((LINUX_LOCK_SH & or)==LINUX_LOCK_SH))
		if_print_or(arg, LINUX_LOCK_SH, LOCK_SH, or);
	if (arg==LINUX_LOCK_EX || !((LINUX_LOCK_EX & or)==LINUX_LOCK_EX))
		if_print_or(arg, LINUX_LOCK_EX, LOCK_EX, or);
	if (arg==LINUX_LOCK_NB || !((LINUX_LOCK_NB & or)==LINUX_LOCK_NB))
		if_print_or(arg, LINUX_LOCK_NB, LOCK_NB, or);
	if (arg==LINUX_LOCK_UN || !((LINUX_LOCK_UN & or)==LINUX_LOCK_UN))
		if_print_or(arg, LINUX_LOCK_UN, LOCK_UN, or);
	if (arg==LINUX_LOCK_MAND || !((LINUX_LOCK_MAND & or)==LINUX_LOCK_MAND))
		if_print_or(arg, LINUX_LOCK_MAND, LOCK_MAND, or);
	if (arg==LINUX_LOCK_READ || !((LINUX_LOCK_READ & or)==LINUX_LOCK_READ))
		if_print_or(arg, LINUX_LOCK_READ, LOCK_READ, or);
	if (arg==LINUX_LOCK_WRITE || !((LINUX_LOCK_WRITE & or)==LINUX_LOCK_WRITE))
		if_print_or(arg, LINUX_LOCK_WRITE, LOCK_WRITE, or);
	if (arg==LINUX_LOCK_RW || !((LINUX_LOCK_RW & or)==LINUX_LOCK_RW))
		if_print_or(arg, LINUX_LOCK_RW, LOCK_RW, or);
	if (or == 0 && or != arg)
		printf("<invalid>%ld", ((long)arg)^or);
}


/*
 * auto-generated from /usr/local/gentoo-stage3/usr/include/asm-generic/mman.h
 */
#define LINUX_PROT_READ		0x1
#define LINUX_PROT_WRITE		0x2
#define LINUX_PROT_EXEC		0x4
#define LINUX_PROT_SEM		0x8
#define LINUX_PROT_NONE		0x0
#define LINUX_PROT_GROWSDOWN		0x01000000
#define LINUX_PROT_GROWSUP		0x02000000



/* AUTO */
void
linux_mmapprotname(int arg)
{
	int	or = 0;
	if (arg==LINUX_PROT_READ || !((LINUX_PROT_READ & or)==LINUX_PROT_READ))
		if_print_or(arg, LINUX_PROT_READ, PROT_READ, or);
	if (arg==LINUX_PROT_WRITE || !((LINUX_PROT_WRITE & or)==LINUX_PROT_WRITE))
		if_print_or(arg, LINUX_PROT_WRITE, PROT_WRITE, or);
	if (arg==LINUX_PROT_EXEC || !((LINUX_PROT_EXEC & or)==LINUX_PROT_EXEC))
		if_print_or(arg, LINUX_PROT_EXEC, PROT_EXEC, or);
	if (arg==LINUX_PROT_SEM || !((LINUX_PROT_SEM & or)==LINUX_PROT_SEM))
		if_print_or(arg, LINUX_PROT_SEM, PROT_SEM, or);
	if (arg==LINUX_PROT_NONE || !((LINUX_PROT_NONE & or)==LINUX_PROT_NONE))
		if_print_or(arg, LINUX_PROT_NONE, PROT_NONE, or);
	if (arg==LINUX_PROT_GROWSDOWN || !((LINUX_PROT_GROWSDOWN & or)==LINUX_PROT_GROWSDOWN))
		if_print_or(arg, LINUX_PROT_GROWSDOWN, PROT_GROWSDOWN, or);
	if (arg==LINUX_PROT_GROWSUP || !((LINUX_PROT_GROWSUP & or)==LINUX_PROT_GROWSUP))
		if_print_or(arg, LINUX_PROT_GROWSUP, PROT_GROWSUP, or);
	if (or == 0 && or != arg)
		printf("<invalid>%ld", ((long)arg)^or);
}


/* AUTO */
void
linux_futexkeyname(int arg)
{
	switch (arg) {
	case 0:
		printf("FUTEX_WAIT");
		break;
	case 1:
		printf("FUTEX_WAKE");
		break;
	case 2:
		printf("FUTEX_FD");
		break;
	case 3:
		printf("FUTEX_REQUEUE");
		break;
	case 4:
		printf("FUTEX_CMP_REQUEUE");
		break;
	case 5:
		printf("FUTEX_WAKE_OP");
		break;
	case 6:
		printf("FUTEX_LOCK_PI");
		break;
	case 7:
		printf("FUTEX_UNLOCK_PI");
		break;
	case 8:
		printf("FUTEX_TRYLOCK_PI");
		break;
	case 128:
		printf("FUTEX_PRIVATE_FLAG");
		break;
	case 0x80000000:
		printf("FUTEX_WAITERS");
		break;
	case 0x40000000:
		printf("FUTEX_OWNER_DIED");
		break;
	case 0x3fffffff:
		printf("FUTEX_TID_MASK");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}


/* AUTO */
void
linux_whencename(int arg)
{
	switch (arg) {
	case 0:
		printf("SEEK_SET");
		break;
	case 1:
		printf("SEEK_CUR");
		break;
	case 2:
		printf("SEEK_END");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}


/* AUTO */
void
linux_madvisebehavname(int arg)
{
	switch (arg) {
	case 0:
		printf("MADV_NORMAL");
		break;
	case 1:
		printf("MADV_RANDOM");
		break;
	case 2:
		printf("MADV_SEQUENTIAL");
		break;
	case 3:
		printf("MADV_WILLNEED");
		break;
	case 4:
		printf("MADV_DONTNEED");
		break;
	case 9:
		printf("MADV_REMOVE");
		break;
	case 10:
		printf("MADV_DONTFORK");
		break;
	case 11:
		printf("MADV_DOFORK");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}


/* AUTO */
void
linux_msyncflagsname(int arg)
{
	switch (arg) {
	case 1:
		printf("MS_ASYNC");
		break;
	case 2:
		printf("MS_INVALIDATE");
		break;
	case 4:
		printf("MS_SYNC");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}


/* AUTO */
void
linux_sigprocmaskhowname(int arg)
{
	switch (arg) {
	case 0:
		printf("SIG_BLOCK");
		break;
	case 1:
		printf("SIG_UNBLOCK");
		break;
	case 2:
		printf("SIG_SETMASK");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}


/*
 * auto-generated from /usr/local/gentoo-stage3/usr/include/linux/stat.h
 */



/* AUTO */
void
linux_modename(int arg)
{
	int	or = 0;
	if (or == 0 && or != arg)
		printf("<invalid>%ld", ((long)arg)^or);
}


/*
 * MANUAL
 */
void
linux_mmapflagsname(int arg)
{
	int or = 0;

	if (!((arg>0)^((0x0100)>0)))
		if_print_or(arg, 0x0100, MAP_GROWSDOWN, or);
	if (!((arg>0)^((0x0800)>0)))
		if_print_or(arg, 0x0800, MAP_DENYWRITE, or);
	if (!((arg>0)^((0x1000)>0)))
		if_print_or(arg, 0x1000, MAP_EXECUTABLE, or);
	if (!((arg>0)^((0x2000)>0)))
		if_print_or(arg, 0x2000, MAP_LOCKED, or);
	if (!((arg>0)^((0x4000)>0)))
		if_print_or(arg, 0x4000, MAP_NORESERVE, or);
	if (!((arg>0)^((0x8000)>0)))
		if_print_or(arg, 0x8000, MAP_POPULATE, or);
	if (!((arg>0)^((0x10000)>0)))
		if_print_or(arg, 0x10000, MAP_NONBLOCK, or);
	if (!((arg>0)^((0x01)>0)))
		if_print_or(arg, 0x01, MAP_SHARED, or);
	if (!((arg>0)^((0x02)>0)))
		if_print_or(arg, 0x02, MAP_PRIVATE, or);
	if (!((arg>0)^((0x0f)>0)))
		if_print_or(arg, 0x0f, MAP_TYPE, or);
	if (!((arg>0)^((0x10)>0)))
		if_print_or(arg, 0x10, MAP_FIXED, or);
	if (!((arg>0)^((0x20)>0)))
		if_print_or(arg, 0x20, MAP_ANONYMOUS, or);
	if (or == 0)
		printf("<invalid>%ld", (long)arg);
}


/*
 * AUTO-Special
 *
 */
void
linux_ptraceopname(int arg)
{

	switch (arg) {
	case 12:
		printf("GETREGS");
		break;
	case 13:
		printf("SETREGS");
		break;
	case 14:
		printf("GETFPREGS");
		break;
	case 15:
		printf("SETFPREGS");
		break;
	case 18:
		printf("GETFPXREGS");
		break;
	case 19:
		printf("SETFPXREGS");
		break;
	case 21:
		printf("OLDSETOPTIONS");
		break;
	case 31:
		printf("SYSEMU");
		break;
	case 0:
		printf("TRACEME");
		break;
	case 1:
		printf("PEEKTEXT");
		break;
	case 2:
		printf("PEEKDATA");
		break;
	case 3:
		printf("PEEKUSR");
		break;
	case 4:
		printf("POKETEXT");
		break;
	case 5:
		printf("POKEDATA");
		break;
	case 6:
		printf("POKEUSR");
		break;
	case 7:
		printf("CONT");
		break;
	case 8:
		printf("KILL");
		break;
	case 9:
		printf("SINGLESTEP");
		break;
	case 16:
		printf("ATTACH");
		break;
	case 17:
		printf("DETACH");
		break;
	case 24:
		printf("SYSCALL");
		break;
	case 0x4200:
		printf("SETOPTIONS");
		break;
	case 0x4201:
		printf("GETEVENTMSG");
		break;
	case 0x4202:
		printf("GETSIGINFO");
		break;
	case 0x4203:
		printf("SETSIGINFO");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}

/*
 * Manual
 */

#define LINUX_PF_UNSPEC		0
#define LINUX_PF_LOCAL		1
#define LINUX_PF_INET		2
#define LINUX_PF_AX25		3
#define LINUX_PF_IPX		4
#define LINUX_PF_APPLETALK		5
#define LINUX_PF_NETROM		6
#define LINUX_PF_BRIDGE		7
#define LINUX_PF_ATMPVC		8
#define LINUX_PF_X25		9
#define LINUX_PF_INET6		10
#define LINUX_PF_ROSE		11
#define LINUX_PF_DECnet		12
#define LINUX_PF_NETBEUI		13
#define LINUX_PF_SECURITY		14
#define LINUX_PF_KEY		15
#define LINUX_PF_NETLINK		16
#define LINUX_PF_PACKET		17
#define LINUX_PF_ASH		18
#define LINUX_PF_ECONET		19
#define LINUX_PF_ATMSVC		20
#define LINUX_PF_SNA		22
#define LINUX_PF_IRDA		23
#define LINUX_PF_PPPOX		24
#define LINUX_PF_WANPIPE		25
#define LINUX_PF_BLUETOOTH		31
#define LINUX_PF_MAX		32
#define	LINUX_PF_UNIX		LINUX_PF_LOCAL
#define	LINUX_PF_FILE		LINUX_PF_LOCAL
#define	LINUX_PF_ROUTE		LINUX_PF_NETLINK

#define	LINUX_AF_UNSPEC	LINUX_PF_UNSPEC
#define	LINUX_AF_LOCAL	LINUX_PF_LOCAL
#define	LINUX_AF_UNIX		LINUX_PF_UNIX
#define	LINUX_AF_FILE		LINUX_PF_FILE
#define	LINUX_AF_INET		LINUX_PF_INET
#define	LINUX_AF_AX25		LINUX_PF_AX25
#define	LINUX_AF_IPX		LINUX_PF_IPX
#define	LINUX_AF_APPLETALK	LINUX_PF_APPLETALK
#define	LINUX_AF_NETROM	LINUX_PF_NETROM
#define	LINUX_AF_BRIDGE	LINUX_PF_BRIDGE
#define	LINUX_AF_ATMPVC	LINUX_PF_ATMPVC
#define	LINUX_AF_X25		LINUX_PF_X25
#define	LINUX_AF_INET6	LINUX_PF_INET6
#define	LINUX_AF_ROSE		LINUX_PF_ROSE
#define	LINUX_AF_DECnet	LINUX_PF_DECnet
#define	LINUX_AF_NETBEUI	LINUX_PF_NETBEUI
#define	LINUX_AF_SECURITY	LINUX_PF_SECURITY
#define	LINUX_AF_KEY		LINUX_PF_KEY
#define	LINUX_AF_NETLINK	LINUX_PF_NETLINK
#define	LINUX_AF_ROUTE	LINUX_PF_ROUTE
#define	LINUX_AF_PACKET	LINUX_PF_PACKET
#define	LINUX_AF_ASH		LINUX_PF_ASH
#define	LINUX_AF_ECONET	LINUX_PF_ECONET
#define	LINUX_AF_ATMSVC	LINUX_PF_ATMSVC
#define	LINUX_AF_SNA		LINUX_PF_SNA
#define	LINUX_AF_IRDA		LINUX_PF_IRDA
#define	LINUX_AF_PPPOX	LINUX_PF_PPPOX
#define	LINUX_AF_WANPIPE	LINUX_PF_WANPIPE
#define	LINUX_AF_BLUETOOTH	LINUX_PF_BLUETOOTH
#define	LINUX_AF_MAX		LINUX_PF_MAX

/*
 * MANUAL
 *
 */
void
linux_sockfamilyname(int arg)
{
	if (arg == LINUX_AF_UNSPEC) 
		printf("AF_UNSPEC");
	else if (arg == LINUX_AF_LOCAL) 
		printf("AF_LOCAL");
	else if (arg == LINUX_AF_UNIX) 
		printf("AF_UNIX");
	else if (arg == LINUX_AF_FILE) 
		printf("AF_FILE");
	else if (arg == LINUX_AF_INET) 
		printf("AF_INET");
	else if (arg == LINUX_AF_AX25) 
		printf("AF_AX25");
	else if (arg == LINUX_AF_IPX) 
		printf("AF_IPX");
	else if (arg == LINUX_AF_APPLETALK) 
		printf("AF_APPLETALK");
	else if (arg == LINUX_AF_NETROM) 
		printf("AF_NETROM");
	else if (arg == LINUX_AF_BRIDGE) 
		printf("AF_BRIDGE");
	else if (arg == LINUX_AF_ATMPVC) 
		printf("AF_ATMPVC");
	else if (arg == LINUX_AF_X25) 
		printf("AF_X25");
	else if (arg == LINUX_AF_INET6) 
		printf("AF_INET6");
	else if (arg == LINUX_AF_ROSE) 
		printf("AF_ROSE");
	else if (arg == LINUX_AF_DECnet) 
		printf("AF_DECnet");
	else if (arg == LINUX_AF_NETBEUI) 
		printf("AF_NETBEUI");
	else if (arg == LINUX_AF_SECURITY) 
		printf("AF_SECURITY");
	else if (arg == LINUX_AF_KEY) 
		printf("AF_KEY");
	else if (arg == LINUX_AF_NETLINK) 
		printf("AF_NETLINK");
	else if (arg == LINUX_AF_ROUTE) 
		printf("AF_ROUTE");
	else if (arg == LINUX_AF_PACKET) 
		printf("AF_PACKET");
	else if (arg == LINUX_AF_ASH) 
		printf("AF_ASH");
	else if (arg == LINUX_AF_ECONET) 
		printf("AF_ECONET");
	else if (arg == LINUX_AF_ATMSVC) 
		printf("AF_ATMSVC");
	else if (arg == LINUX_AF_SNA) 
		printf("AF_SNA");
	else if (arg == LINUX_AF_IRDA) 
		printf("AF_IRDA");
	else if (arg == LINUX_AF_PPPOX) 
		printf("AF_PPPOX");
	else if (arg == LINUX_AF_WANPIPE) 
		printf("AF_WANPIPE");
	else if (arg == LINUX_AF_BLUETOOTH) 
		printf("AF_BLUETOOTH");
	else if (arg == LINUX_AF_MAX) 
		printf("AF_MAX");
	else /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
}


/*
 * MANUAL
 *
 */
void
linux_sockdomainname(int arg)
{
	if (arg == LINUX_PF_UNSPEC) 
		printf("PF_UNSPEC");
	else if (arg == LINUX_PF_LOCAL) 
		printf("PF_LOCAL");
	else if (arg == LINUX_PF_UNIX) 
		printf("PF_UNIX");
	else if (arg == LINUX_PF_FILE) 
		printf("PF_FILE");
	else if (arg == LINUX_PF_INET) 
		printf("PF_INET");
	else if (arg == LINUX_PF_AX25) 
		printf("PF_AX25");
	else if (arg == LINUX_PF_IPX) 
		printf("PF_IPX");
	else if (arg == LINUX_PF_APPLETALK) 
		printf("PF_APPLETALK");
	else if (arg == LINUX_PF_NETROM) 
		printf("PF_NETROM");
	else if (arg == LINUX_PF_BRIDGE) 
		printf("PF_BRIDGE");
	else if (arg == LINUX_PF_ATMPVC) 
		printf("PF_ATMPVC");
	else if (arg == LINUX_PF_X25) 
		printf("PF_X25");
	else if (arg == LINUX_PF_INET6) 
		printf("PF_INET6");
	else if (arg == LINUX_PF_ROSE) 
		printf("PF_ROSE");
	else if (arg == LINUX_PF_DECnet) 
		printf("PF_DECnet");
	else if (arg == LINUX_PF_NETBEUI) 
		printf("PF_NETBEUI");
	else if (arg == LINUX_PF_SECURITY) 
		printf("PF_SECURITY");
	else if (arg == LINUX_PF_KEY) 
		printf("PF_KEY");
	else if (arg == LINUX_PF_NETLINK) 
		printf("PF_NETLINK");
	else if (arg == LINUX_PF_ROUTE) 
		printf("PF_ROUTE");
	else if (arg == LINUX_PF_PACKET) 
		printf("PF_PACKET");
	else if (arg == LINUX_PF_ASH) 
		printf("PF_ASH");
	else if (arg == LINUX_PF_ECONET) 
		printf("PF_ECONET");
	else if (arg == LINUX_PF_ATMSVC) 
		printf("PF_ATMSVC");
	else if (arg == LINUX_PF_SNA) 
		printf("PF_SNA");
	else if (arg == LINUX_PF_IRDA) 
		printf("PF_IRDA");
	else if (arg == LINUX_PF_PPPOX) 
		printf("PF_PPPOX");
	else if (arg == LINUX_PF_WANPIPE) 
		printf("PF_WANPIPE");
	else if (arg == LINUX_PF_BLUETOOTH) 
		printf("PF_BLUETOOTH");
	else if (arg == LINUX_PF_MAX) 
		printf("PF_MAX");
	else /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
}


/*
 * auto-generated from /usr/local/gentoo-stage3/usr/include/linux/net.h
 */
#define LINUX_SYS_SOCKET		1
#define LINUX_SYS_BIND		2
#define LINUX_SYS_CONNECT		3
#define LINUX_SYS_LISTEN		4
#define LINUX_SYS_ACCEPT		5
#define LINUX_SYS_GETSOCKNAME		6
#define LINUX_SYS_GETPEERNAME		7
#define LINUX_SYS_SOCKETPAIR		8
#define LINUX_SYS_SEND		9
#define LINUX_SYS_RECV		10
#define LINUX_SYS_SENDTO		11
#define LINUX_SYS_RECVFROM		12
#define LINUX_SYS_SHUTDOWN		13
#define LINUX_SYS_SETSOCKOPT		14
#define LINUX_SYS_GETSOCKOPT		15
#define LINUX_SYS_SENDMSG		16
#define LINUX_SYS_RECVMSG		17



/*
 * AUTO-Special
 *
 */
void
linux_sockcallname(int arg)
{

	switch (arg) {
	case LINUX_SYS_SOCKET:
		printf("SOCKET");
		break;
	case LINUX_SYS_BIND:
		printf("BIND");
		break;
	case LINUX_SYS_CONNECT:
		printf("CONNECT");
		break;
	case LINUX_SYS_LISTEN:
		printf("LISTEN");
		break;
	case LINUX_SYS_ACCEPT:
		printf("ACCEPT");
		break;
	case LINUX_SYS_GETSOCKNAME:
		printf("GETSOCKNAME");
		break;
	case LINUX_SYS_GETPEERNAME:
		printf("GETPEERNAME");
		break;
	case LINUX_SYS_SOCKETPAIR:
		printf("SOCKETPAIR");
		break;
	case LINUX_SYS_SEND:
		printf("SEND");
		break;
	case LINUX_SYS_RECV:
		printf("RECV");
		break;
	case LINUX_SYS_SENDTO:
		printf("SENDTO");
		break;
	case LINUX_SYS_RECVFROM:
		printf("RECVFROM");
		break;
	case LINUX_SYS_SHUTDOWN:
		printf("SHUTDOWN");
		break;
	case LINUX_SYS_SETSOCKOPT:
		printf("SETSOCKOPT");
		break;
	case LINUX_SYS_GETSOCKOPT:
		printf("GETSOCKOPT");
		break;
	case LINUX_SYS_SENDMSG:
		printf("SENDMSG");
		break;
	case LINUX_SYS_RECVMSG:
		printf("RECVMSG");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}


/*
 * auto-generated from /usr/local/gentoo-stage3/usr/include/linux/wait.h
 */
#define LINUX_WNOHANG		0x00000001
#define LINUX_WUNTRACED		0x00000002
#define LINUX_WEXITED		0x00000004
#define LINUX_WCONTINUED		0x00000008
#define LINUX_WNOWAIT		0x01000000
#define LINUX___WNOTHREAD		0x20000000
#define LINUX___WALL		0x40000000
#define LINUX___WCLONE		0x80000000



/*
 * MANUAL, 0 in options valid
 *
 */
void
linux_waitoptname(int arg)
{
	int or = 0;

	if (0 == arg)
	{
		printf("options=0");
		return;
	}

	if (!((arg>0)^((LINUX_WNOHANG)>0)))
		if_print_or(arg, LINUX_WNOHANG, WNOHANG, or);
	if (!((arg>0)^((LINUX_WUNTRACED)>0)))
		if_print_or(arg, LINUX_WUNTRACED, WUNTRACED, or);
	if (!((arg>0)^((LINUX_WEXITED)>0)))
		if_print_or(arg, LINUX_WEXITED, WEXITED, or);
	if (!((arg>0)^((LINUX_WCONTINUED)>0)))
		if_print_or(arg, LINUX_WCONTINUED, WCONTINUED, or);
	if (!((arg>0)^((LINUX_WNOWAIT)>0)))
		if_print_or(arg, LINUX_WNOWAIT, WNOWAIT, or);
	if (!((arg>0)^((LINUX___WNOTHREAD)>0)))
		if_print_or(arg, LINUX___WNOTHREAD, __WNOTHREAD, or);
	if (!((arg>0)^((LINUX___WALL)>0)))
		if_print_or(arg, LINUX___WALL, __WALL, or);
	if (!((arg>0)^((LINUX___WCLONE)>0)))
		if_print_or(arg, LINUX___WCLONE, __WCLONE, or);
	if (or == 0)
		printf("<invalid>%ld", (long)arg);
}

/*
 * MANUAL
 *
 */
void
linux_flagsandmodename(int flags, int mode, int decimal)
{
	linux_flagsname(flags);
	(void)putchar(',');

	if ((flags & LINUX_O_CREAT) == LINUX_O_CREAT)
		linux_modename(mode);
	else {
		if (decimal)
			printf("<unused>%ld", (long)mode);
		else
			printf("<unused>%#lx", (long)mode);
	}
}


/*
 * auto-generated defines from /usr/local/gentoo-stage3/usr/include/asm-generic/fcntl.h
 */

#define LINUX_F_DUPFD	 0 /* dup */
#define LINUX_F_GETFD	 1 /* get close_on_exec */
#define LINUX_F_SETFD	 2 /* set/clear close_on_exec */
#define LINUX_F_GETFL	 3 /* get file->f_flags */
#define LINUX_F_SETFL	 4 /* set file->f_flags */
#define LINUX_F_GETLK	 5
#define LINUX_F_SETLK	 6
#define LINUX_F_SETLKW	 7
#define LINUX_F_SETOWN	 8 /* for sockets. */
#define LINUX_F_GETOWN	 9 /* for sockets. */
#define LINUX_F_SETSIG	 10 /* for sockets. */
#define LINUX_F_GETSIG	 11 /* for sockets. */
#define LINUX_FD_CLOEXEC	 1 /* actually anything with low bit set goes */
#define LINUX_F_GETLK64	 12 /* using 'struct flock64' */
#define LINUX_F_SETLK64	 13
#define LINUX_F_SETLKW64	 14

/*
 * AUTO - Special
 * F_ is used to specify fcntl commands as well as arguments. Both sets are
 * grouped in asm-generic/fcntl.h, and this awk script grabs the first group.
 */
void
linux_fcntlcmdname(int cmd, int arg, int decimal)
{
	switch (cmd) {
	case LINUX_F_DUPFD:
		printf("F_DUPFD");
		break;
	case LINUX_F_GETFD:
		printf("F_GETFD");
		break;
	case LINUX_F_SETFD:
		printf("F_SETFD");
		break;
	case LINUX_F_GETFL:
		printf("F_GETFL");
		break;
	case LINUX_F_SETFL:
		printf("F_SETFL");
		break;
	case LINUX_F_GETLK:
		printf("F_GETLK");
		break;
	case LINUX_F_SETLK:
		printf("F_SETLK");
		break;
	case LINUX_F_SETLKW:
		printf("F_SETLKW");
		break;
	case LINUX_F_SETOWN:
		printf("F_SETOWN");
		break;
	case LINUX_F_GETOWN:
		printf("F_GETOWN");
		break;
	case LINUX_F_SETSIG:
		printf("F_SETSIG");
		break;
	case LINUX_F_GETSIG:
		printf("F_GETSIG");
		break;
	case LINUX_F_GETLK64:
		printf("F_GETLK64");
		break;
	case LINUX_F_SETLK64:
		printf("F_SETLK64");
		break;
	case LINUX_F_SETLKW64:
		printf("F_SETLKW64");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)cmd);
	}
	(void)putchar(',');
	if (cmd == LINUX_F_GETFD || cmd == LINUX_F_SETFD) {
		if (arg == LINUX_FD_CLOEXEC)
			printf("FD_CLOEXEC");
		else if (arg == 0)
			printf("0");
		else {
			if (decimal)
				printf("<invalid>%ld", (long)arg);
			else
				printf("<invalid>%#lx", (long)arg);
		}
	} else if (cmd == LINUX_F_SETFL) {
		linux_flagsname(arg);
	} else {
		if (decimal)
			printf("%ld", (long)arg);
		else
			printf("%#lx", (long)arg);
	}
}


/*
 * SystemV IPC defines
  */
#define LINUX_SEMOP		1
#define LINUX_SEMGET		2
#define LINUX_SEMCTL		3
#define LINUX_SEMTIMEDOP	4

#define LINUX_MSGSND		11
#define LINUX_MSGRCV		12
#define LINUX_MSGGET		13
#define LINUX_MSGCTL		14

#define LINUX_MSG_STAT		11
#define LINUX_MSG_INFO		12

#define LINUX_MSG_NOERROR	010000
#define LINUX_MSG_EXCEPT	020000

#define LINUX_SHMAT		21
#define LINUX_SHMDT		22
#define LINUX_SHMGET		23
#define LINUX_SHMCTL		24

#define LINUX_IPC_RMID		0
#define LINUX_IPC_SET		1
#define LINUX_IPC_STAT		2
#define LINUX_IPC_INFO		3

#define LINUX_SHM_LOCK		11
#define LINUX_SHM_UNLOCK	12
#define LINUX_SHM_STAT		13
#define LINUX_SHM_INFO		14
#define LINUX_SHM_RDONLY	0x1000
#define LINUX_SHM_RND		0x2000
#define LINUX_SHM_REMAP		0x4000
#define LINUX_SHM_EXEC		0x10000

#define LINUX_SHM_R		0400
#define LINUX_SHM_W		0200

#define LINUX_IPC_CREAT		01000
#define LINUX_IPC_EXCL		02000
#define LINUX_IPC_NOWAIT	04000
#define LINUX_IPC_DIPC		00010000
#define LINUX_IPC_OWN		00020000
#define LINUX_IPC_64		0x0100

/* semctl commands */
#define LINUX_GETPID            11
#define LINUX_GETVAL            12
#define LINUX_GETALL            13
#define LINUX_GETNCNT           14
#define LINUX_GETZCNT           15
#define LINUX_SETVAL            16
#define LINUX_SETALL            17
#define LINUX_SEM_STAT          18
#define LINUX_SEM_INFO          19

/*
 * MANUAL
 * F_ linux_ipc call argument print
 * retval=count of arg to process, or -1 if it is unknown
 */
int
linux_ipccallname(int call)
{
	int ret = -1;

	switch (call&0xffff) {
	case LINUX_SEMOP:
		printf("SEMOP");
		break;
	case LINUX_SEMGET:
		printf("SEMGET");
		break;
	case LINUX_SEMCTL:
		printf("SEMCTL");
		break;
	case LINUX_SEMTIMEDOP:
		printf("SEMTIMEDOP");
		break;
	case LINUX_MSGSND:
		printf("MSGSND");
		break;
	case LINUX_MSGRCV:
		printf("MSGRCV");
		break;
	case LINUX_MSGGET:
		printf("MSGGET");
		break;
	case LINUX_MSGCTL:
		printf("MSGCTL");
		break;
	case LINUX_SHMAT:
		printf("SHMAT");
		break;
	case LINUX_SHMDT:
		printf("SHMDT");
		break;
	case LINUX_SHMGET:
		printf("SHMGET");
		break;
	case LINUX_SHMCTL:
		printf("SHMCTL");
		break;
	default:
		printf("<invalid=%d>", call);
	}

	switch (call&0xffff) {
	case LINUX_SEMOP:
		ret = 3;
		break;
	case LINUX_SEMGET:
		ret = 3;
		break;
	case LINUX_SEMCTL:
		ret = 4;
		break;
	case LINUX_SEMTIMEDOP:
		ret = 4;
		break;
	case LINUX_MSGSND:
		ret = 4;
		break;
	case LINUX_MSGRCV:
		ret = 5;
		break;
	case LINUX_MSGGET:
		ret = 2;
		break;
	case LINUX_MSGCTL:
		ret = 3;
		break;
	case LINUX_SHMAT:
		ret = 3;
		break;
	case LINUX_SHMDT:
		ret = 1;
		break;
	case LINUX_SHMGET:
		ret = 3;
		break;
	case LINUX_SHMCTL:
		ret = 3;
		break;
	default:
		ret=-1;
	}

	return(ret);
}


/*
 * MANUAL
 * F_ linux_ipc 2 arg print
 *
 */
void
linux_ipc2arg(int call, long* arg, int decimal)
{

	switch (call&0xffff) {
	case LINUX_SEMGET:
	case LINUX_MSGGET:
	case LINUX_SHMGET:
		if (*arg == LINUX_IPC_PRIVATE)
		{
			printf("IPC_PRIVATE");
			return;
		}
	default:
		if (decimal)
			printf("%ld", (long)*arg);
		else
			printf("%#lx", (long)*arg);
	}
}


/*
 * MANUAL
 * F_ linux_ipc 3 arg print
 *
 */
void
linux_ipc3arg(int call, long* arg, int decimal)
{
	int or = 0;
	int cmd = *arg;

	switch (call&0xffff) {
	case LINUX_MSGGET:
		if (!((*arg>0)^((LINUX_IPC_CREAT)>0)))
			if_print_or(*arg, LINUX_IPC_CREAT, IPC_CREAT, or);
		if (!((*arg>0)^((LINUX_IPC_EXCL)>0)))
			if_print_or(*arg, LINUX_IPC_EXCL, IPC_EXCL, or);
		if (!((*arg>0)^((LINUX_IPC_NOWAIT)>0)))
			if_print_or(*arg, LINUX_IPC_NOWAIT, IPC_NOWAIT, or);
		if (!((*arg>0)^((LINUX_IPC_DIPC)>0)))
			if_print_or(*arg, LINUX_IPC_DIPC, IPC_DIPC, or);
		if (or == 0)
			printf("%#lx", (long)*arg);
		break;
	case LINUX_MSGCTL:
		if (*arg & LINUX_IPC_64)
			cmd^=LINUX_IPC_64;
		switch(cmd) {
		case LINUX_IPC_RMID:
			printf("IPC_RMID");
			break;
		case LINUX_IPC_SET:
			printf("IPC_SET");
			break;
		case LINUX_IPC_STAT:
			printf("IPC_STAT");
			break;
		case LINUX_IPC_INFO:
			printf("IPC_INFO");
			break;
		case LINUX_MSG_STAT:
			printf("MSG_STAT");
			break;
		case LINUX_MSG_INFO:
			printf("MSG_INFO");
			break;
		default:
			if (decimal)
				printf("<invalid>%ld", (long)cmd);
			else
				printf("<invalid>%#lx", (long)cmd);
		}
		break;
	case LINUX_SHMCTL:
		if (*arg & LINUX_IPC_64)
			cmd^=LINUX_IPC_64;
		else
			cmd=*arg;
		switch(cmd) {
		case LINUX_IPC_RMID:
			printf("IPC_RMID");
			break;
		case LINUX_IPC_SET:
			printf("IPC_SET");
			break;
		case LINUX_IPC_STAT:
			printf("IPC_STAT");
			break;
		case LINUX_IPC_INFO:
			printf("IPC_INFO");
			break;
		case LINUX_SHM_STAT:
			printf("SHM_STAT");
			break;
		case LINUX_SHM_INFO:
			printf("SHM_INFO");
			break;
		case LINUX_SHM_LOCK:
			printf("SHM_LOCK");
			break;
		case LINUX_SHM_UNLOCK:
			printf("SHM_UNLOCK");
			break;
		default:
			if (decimal)
				printf("<invalid>%ld", (long)cmd);
			else
				printf("<invalid>%#lx", (long)cmd);
		}
		break;
	default:
		if (decimal)
			printf("%ld", (long)*arg);
		else
			printf("%#lx", (long)*arg);
	}
}


/*
 * MANUAL
 * F_ linux_ipc 4 arg print
 *
 */
void
linux_ipc4arg(int call, long* arg, int decimal)
{
	int	or = 0;
	int	cmd = *arg;

	switch (call&0xffff) {
	case LINUX_SHMGET:
		if (!((*arg>0)^((LINUX_SHM_R)>0)))
			if_print_or(*arg, LINUX_SHM_R, SHM_R, or);
		if (!((*arg>0)^((LINUX_SHM_W)>0)))
			if_print_or(*arg, LINUX_SHM_W, SHM_W, or);
	case LINUX_SEMGET:
		if (!((*arg>0)^((LINUX_IPC_CREAT)>0)))
			if_print_or(*arg, LINUX_IPC_CREAT, IPC_CREAT, or);
		if (!((*arg>0)^((LINUX_IPC_EXCL)>0)))
			if_print_or(*arg, LINUX_IPC_EXCL, IPC_EXCL, or);
		if (!((*arg>0)^((LINUX_IPC_NOWAIT)>0)))
			if_print_or(*arg, LINUX_IPC_NOWAIT, IPC_NOWAIT, or);
		if (!((*arg>0)^((LINUX_IPC_DIPC)>0)))
			if_print_or(*arg, LINUX_IPC_DIPC, IPC_DIPC, or);
		if (or == 0)
			printf("%#lx", (long)*arg);
		break;
	case LINUX_SEMCTL:
		if (*arg & LINUX_IPC_64)
			cmd^=LINUX_IPC_64;
		switch(cmd) {
		case LINUX_IPC_RMID:
			printf("IPC_RMID");
			break;
		case LINUX_IPC_SET:
			printf("IPC_SET");
			break;
		case LINUX_IPC_STAT:
			printf("IPC_STAT");
			break;
		case LINUX_IPC_INFO:
			printf("IPC_INFO");
			break;
		case LINUX_SEM_STAT:
			printf("SEM_STAT");
			break;
		case LINUX_SEM_INFO:
			printf("SEM_INFO");
			break;
		case LINUX_GETPID:
			printf("GETPID");
			break;
		case LINUX_GETVAL:
			printf("GERVAL");
			break;
		case LINUX_GETALL:
			printf("GETALL");
			break;
		case LINUX_GETNCNT:
			printf("GETNCNT");
			break;
		case LINUX_GETZCNT:
			printf("GETZCNT");
			break;
		case LINUX_SETVAL:
			printf("SETVAL");
			break;
		case LINUX_SETALL:
			printf("SETALL");
			break;
		default:
			if (decimal)
				printf("<invalid>%ld", (long)cmd);
			else
				printf("<invalid>%#lx", (long)cmd);
		}
		break;
	case LINUX_SHMAT:
		if (!((*arg>0)^((LINUX_SHM_RDONLY)>0)))
			if_print_or(*arg, LINUX_SHM_RDONLY, SHM_RDONLY, or);
		if (!((*arg>0)^((LINUX_SHM_RND)>0)))
			if_print_or(*arg, LINUX_SHM_RND, SHM_RND, or);
		if (!((*arg>0)^((LINUX_SHM_REMAP)>0)))
			if_print_or(*arg, LINUX_SHM_REMAP, SHM_REMAP, or);
		if (!((*arg>0)^((LINUX_SHM_EXEC)>0)))
			if_print_or(*arg, LINUX_SHM_EXEC, SHM_EXEC, or);
		if (or == 0)
			printf("%#lx", (long)*arg);
		break;
	default:
		if (decimal)
			printf("%ld", (long)*arg);
		else
			printf("%#lx", (long)*arg);
	}
}


/*
 * MANUAL
 * F_ linux_ipc 5 arg print
 *
 */
void
linux_ipc5arg(int call, long* arg, int decimal)
{
	int	or = 0;

	switch (call&0xffff) {
	case LINUX_MSGSND:
		if (!((*arg>0)^((LINUX_IPC_CREAT)>0)))
			if_print_or(*arg, LINUX_IPC_CREAT, IPC_CREAT, or);
		if (!((*arg>0)^((LINUX_IPC_EXCL)>0)))
			if_print_or(*arg, LINUX_IPC_EXCL, IPC_EXCL, or);
		if (!((*arg>0)^((LINUX_IPC_NOWAIT)>0)))
			if_print_or(*arg, LINUX_IPC_NOWAIT, IPC_NOWAIT, or);
		if (!((*arg>0)^((LINUX_IPC_DIPC)>0)))
			if_print_or(*arg, LINUX_IPC_DIPC, IPC_DIPC, or);
		if (!((*arg>0)^((LINUX_MSG_NOERROR)>0)))
			if_print_or(*arg, LINUX_MSG_NOERROR, MSG_NOERROR, or);
		if (!((*arg>0)^((LINUX_MSG_EXCEPT)>0)))
			if_print_or(*arg, LINUX_MSG_EXCEPT, MSG_EXCEPT, or);
		if (or == 0)
			printf("%#lx", (long)*arg);
		break;
	default:
		if (decimal)
			printf("%ld", (long)*arg);
		else
			printf("%#lx", (long)*arg);
	}
}


/*
 * MANUAL
 * F_ linux_ipc 6 arg print
 *
 */
void
linux_ipc6arg(int call, long* arg, int decimal)
{
	int	or = 0;

	switch (call&0xffff) {
	case LINUX_MSGRCV:
		if (!((*arg>0)^((LINUX_IPC_CREAT)>0)))
			if_print_or(*arg, LINUX_IPC_CREAT, IPC_CREAT, or);
		if (!((*arg>0)^((LINUX_IPC_EXCL)>0)))
			if_print_or(*arg, LINUX_IPC_EXCL, IPC_EXCL, or);
		if (!((*arg>0)^((LINUX_IPC_NOWAIT)>0)))
			if_print_or(*arg, LINUX_IPC_NOWAIT, IPC_NOWAIT, or);
		if (!((*arg>0)^((LINUX_IPC_DIPC)>0)))
			if_print_or(*arg, LINUX_IPC_DIPC, IPC_DIPC, or);
		if (!((*arg>0)^((LINUX_MSG_NOERROR)>0)))
			if_print_or(*arg, LINUX_MSG_NOERROR, MSG_NOERROR, or);
		if (!((*arg>0)^((LINUX_MSG_EXCEPT)>0)))
			if_print_or(*arg, LINUX_MSG_EXCEPT, MSG_EXCEPT, or);
		if (or == 0)
			printf("%#lx", (long)*arg);
		break;
	default:
		if (decimal)
			printf("%ld", (long)*arg);
		else
			printf("%#lx", (long)*arg);
	}
}


/*
 * MANUAL
 * F_ signal - print default sighandlers
 *
 */
void
linux_sighandlername(long arg, int decimal)
{

	if (arg == (long)LINUX_SIG_DFL)
		printf("SIG_DFL");
	else if (arg == (long)LINUX_SIG_IGN)
		printf("SIG_IGN");
	else {
		if (decimal)
			printf("%ld", (long)arg);
		else
			printf("%#lx", (long)arg);
	}
}


/*
 * MANUAL
 * F_ - print dfd parameter
 *
 */
void
linux_openatdfdname(int arg, int decimal)
{
	if (arg == LINUX_AT_FDCWD)
		printf("AT_FDCWD");
	else {
		if (decimal)
			printf("%ld", (long)arg);
		else
			printf("%#lx", (long)arg);
	}
}

void
linux_newatflagsname(int arg, int decimal)
{
	if (arg == LINUX_AT_SYMLINK_NOFOLLOW)
		printf("AT_SYMLINK_NOFOLLOW");
	else if (arg == LINUX_AT_REMOVEDIR)
		printf("AT_REMOVEDIR");
	else if (arg == LINUX_AT_SYMLINK_FOLLOW)
		printf("AT_SYMLINK_FOLLOW");
	else {
		if (decimal)
			printf("<invalid>%ld", (long)arg);
		else
			printf("<invalid>%#lx", (long)arg);
	}
}


/*
 * auto-generated from /usr/local/gentoo-stage3/usr/include/unistd.h
 */
#define LINUX_R_OK		4
#define LINUX_W_OK		2
#define LINUX_X_OK		1
#define LINUX_F_OK		0



/* AUTO */
void
linux_accessmodename(int arg)
{
	int	or = 0;
	if (arg==LINUX_R_OK || !((LINUX_R_OK & or)==LINUX_R_OK))
		if_print_or(arg, LINUX_R_OK, R_OK, or);
	if (arg==LINUX_W_OK || !((LINUX_W_OK & or)==LINUX_W_OK))
		if_print_or(arg, LINUX_W_OK, W_OK, or);
	if (arg==LINUX_X_OK || !((LINUX_X_OK & or)==LINUX_X_OK))
		if_print_or(arg, LINUX_X_OK, X_OK, or);
	if (arg==LINUX_F_OK || !((LINUX_F_OK & or)==LINUX_F_OK))
		if_print_or(arg, LINUX_F_OK, F_OK, or);
	if (or == 0 && or != arg)
		printf("<invalid>%ld", ((long)arg)^or);
}

/*
 * auto-generated from /usr/local/gentoo-stage3/usr/include/bits/socket.h
 */
#define LINUX_SOCK_STREAM		1
#define LINUX_SOCK_DGRAM		2
#define LINUX_SOCK_RAW		3
#define LINUX_SOCK_RDM		4
#define LINUX_SOCK_SEQPACKET		5
#define LINUX_SOCK_PACKET		10



/* AUTO */
void
linux_socktypename(int arg)
{
	switch (arg) {
	case LINUX_SOCK_STREAM:
		printf("SOCK_STREAM");
		break;
	case LINUX_SOCK_DGRAM:
		printf("SOCK_DGRAM");
		break;
	case LINUX_SOCK_RAW:
		printf("SOCK_RAW");
		break;
	case LINUX_SOCK_RDM:
		printf("SOCK_RDM");
		break;
	case LINUX_SOCK_SEQPACKET:
		printf("SOCK_SEQPACKET");
		break;
	case LINUX_SOCK_PACKET:
		printf("SOCK_PACKET");
		break;
	default: /* Should not reach */
		printf("<invalid=%ld>", (long)arg);
	}
}


/*
 * auto-generated from /usr/local/gentoo-stage3/usr/include/linux/futex.h
 */
#define LINUX_FUTEX_WAIT		0
#define LINUX_FUTEX_WAKE		1
#define LINUX_FUTEX_FD		2
#define LINUX_FUTEX_REQUEUE		3
#define LINUX_FUTEX_CMP_REQUEUE		4
#define LINUX_FUTEX_WAKE_OP		5
#define LINUX_FUTEX_LOCK_PI		6
#define LINUX_FUTEX_UNLOCK_PI		7
#define LINUX_FUTEX_TRYLOCK_PI		8
#define LINUX_FUTEX_PRIVATE_FLAG		128
#define LINUX_FUTEX_WAITERS		0x80000000
#define LINUX_FUTEX_OWNER_DIED		0x40000000
#define LINUX_FUTEX_TID_MASK		0x3fffffff



/*
 * MANUAL
 *
 */
void
linux_futexopname(int arg)
{

#ifdef LINUX_FUTEX_PRIVATE_FLAG

	if (arg & LINUX_FUTEX_PRIVATE_FLAG) {
		printf("FUTEX_PRIVATE_FLAG|");
		arg &= ~LINUX_FUTEX_PRIVATE_FLAG;
	}
#endif

	linux_futexkeyname(arg);
}

