/*-
 * Copyright (c) 1988, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

#define _KERNEL
extern int errno;
#include <sys/errno.h>
#undef _KERNEL
#include <sys/param.h>
#include <sys/uio.h>
#include <sys/ktrace.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/signalvar.h>
#include <sys/un.h>

#include <netinet/in.h>
#include <arpa/inet.h>

#ifdef __amd64_
#include <amd64/linux32/linux.h>
#else
#include <i386/linux/linux.h>
#endif

#include <ctype.h>
#include <grp.h>
#include <inttypes.h>
#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "kdump.h"
#include "kdump_subr.h"
#include "linux_syscall.h"

extern void linux_ioctlname(unsigned long, int);

extern int timestamp, decimal, fancy, suppressdata, tail, threads,
    maxdata, resolv;

extern int nlinux_syscalls;
extern char *linux_syscallnames[];
extern int nlinux_errors;
extern struct linux_error_msg linux_errors[];
extern char* linux_signames[];
extern int nlinux_signames;

/*
 * from linux.h
 * Linux syscalls return negative errno's, we do positive and map them
 */
static int bsd_to_linux_errno[ELAST + 1] = {
	-0,  -1,  -2,  -3,  -4,  -5,  -6,  -7,  -8,  -9,
	-10, -35, -12, -13, -14, -15, -16, -17, -18, -19,
	-20, -21, -22, -23, -24, -25, -26, -27, -28, -29,
	-30, -31, -32, -33, -34, -11,-115,-114, -88, -89,
	-90, -91, -92, -93, -94, -95, -96, -97, -98, -99,
	-100,-101,-102,-103,-104,-105,-106,-107,-108,-109,
	-110,-111, -40, -36,-112,-113, -39, -11, -87,-122,
	-116, -66,  -6,  -6,  -6,  -6,  -6, -37, -38,  -9,
	-6,  -6, -43, -42, -75,-125, -84, -95, -16, -74,
	-72, -67, -71
};

void
linux_ktrsyscall(struct ktr_syscall *ktr)
{
	register int narg = ktr->ktr_narg;
	register register_t *ip;

	if (ktr->ktr_code >= nlinux_syscalls || ktr->ktr_code < 0)
		printf("[%d]", ktr->ktr_code);
	else
		printf("%s", linux_syscallnames[ktr->ktr_code]);
	ip = &ktr->ktr_args[0];
	if (narg) {
		char c = '(';
		if (fancy) {

			if (ktr->ktr_code == LINUX_SYS_linux_ioctl) {
				print_number(ip,narg,c);
				putchar(',');
				linux_ioctlname((u_long)*ip, decimal);
				c = ',';
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_access) {
				print_number(ip,narg,c);
				putchar(',');
				linux_accessmodename((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_open) {
				int flags;
				int mode;
				print_number(ip,narg,c);
				flags = *ip;
				mode = *++ip;
				putchar(',');
				linux_flagsandmodename(flags, mode, decimal);
				ip++;
				narg-=2;
			} else if (ktr->ktr_code == LINUX_SYS_linux_mmap2) {
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				putchar(',');
				linux_mmapprotname((int)*ip);
				putchar(',');
				ip++;
				narg--;
				linux_mmapflagsname((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_mprotect) {
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				putchar(',');
				linux_mmapprotname((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_sigaction ||
				ktr->ktr_code == LINUX_SYS_linux_rt_sigaction) {
				putchar('(');
				linux_signame((int)*ip);
				c=',';
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_sigprocmask ||
				ktr->ktr_code == LINUX_SYS_linux_rt_sigprocmask) {
				putchar('(');
				linux_sigprocmaskhowname((int)*ip);
				ip++;
				narg--;
				c=',';
			} else if (ktr->ktr_code == LINUX_SYS_linux_fcntl ||
				ktr->ktr_code == LINUX_SYS_linux_fcntl64) {
				int cmd;
				int arg;
				print_number(ip,narg,c);
				cmd = *ip;
				arg = *++ip;
				putchar(',');
				linux_fcntlcmdname(cmd, arg, decimal);
				ip++;
				narg-=2;
			} else if (ktr->ktr_code == LINUX_SYS_linux_lseek) {
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				putchar(',');
				linux_whencename((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_llseek) {
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				putchar(',');
				linux_whencename((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_socketcall) {
				putchar('(');
				linux_sockcallname((int)*ip);
				c=',';
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_wait4 ||
				ktr->ktr_code == LINUX_SYS_linux_waitpid) {
				/* pid as decimal */
				(void)printf("(%d", (int)*ip);
				ip++;
				narg--;
				c=',';
				print_number(ip,narg,c);
				putchar(',');
				linux_waitoptname((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_umask &&
				    (int)*ip > 0) {
				putchar('(');
				linux_modename((int)*ip);
				c=',';
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_getrusage) {
				putchar('(');
				linux_rusagename((int)*ip);
				ip++;
				narg--;
				c=',';
			} else if (ktr->ktr_code == LINUX_SYS_linux_sched_get_priority_max ||
				ktr->ktr_code == LINUX_SYS_linux_sched_get_priority_min) {
				putchar('(');
				linux_schedpolicyname((int)*ip);
				c=',';
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_kill ||
				ktr->ktr_code == LINUX_SYS_linux_tkill) {
				/* pid, tid are decimal */
				(void)printf("(%d,", (int)*ip);
				ip++;
				narg--;
				linux_signame((int)*ip);
				c=',';
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_tgkill) {
				/* pid, tid are decimal */
				(void)printf("(%d", (int)*ip);
				(void)printf(",%d,", (int)*++ip);
				ip++;
				narg-=2;
				linux_signame((int)*ip);
				c=',';
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_chmod) {
				print_number(ip,narg,c);
				putchar(',');
				linux_modename((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_mknod) {
				print_number(ip,narg,c);
				putchar(',');
				linux_modename((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_mkdir ||
				ktr->ktr_code == LINUX_SYS_linux_creat) {
				print_number(ip,narg,c);
				putchar(',');
				linux_modename((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_setrlimit ||
				ktr->ktr_code == LINUX_SYS_linux_getrlimit) {
				putchar('(');
				linux_rlimitname((int)*ip);
				c=',';
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_ipc) {
				int ncall;
				int icall;
				ncall = *ip;
				putchar('(');
				icall = linux_ipccallname(ncall);
				ip++;
				narg--;
				c=',';
				if (narg > 0 && icall > 0) {
					putchar(',');
					linux_ipc2arg(ncall, (long *)ip, decimal);
					ip++;
					narg--;
				}
				if (narg > 0 && icall > 1) {
					putchar(',');
					linux_ipc3arg(ncall, (long *)ip, decimal);
					ip++;
					narg--;
				}
				if (narg > 0 && icall > 2) {
					putchar(',');
					linux_ipc4arg(ncall, (long *)ip, decimal);
					ip++;
					narg--;
				}
				if (narg > 0 && icall > 3) {
					putchar(',');
					linux_ipc5arg(ncall, (long *)ip, decimal);
					ip++;
					narg--;
				}
				if (narg > 0 && icall > 4) {
					putchar(',');
					linux_ipc6arg(ncall, (long *)ip, decimal);
					ip++;
					narg--;
				}
				narg=0;
			} else if (ktr->ktr_code == LINUX_SYS_linux_sys_futex) {
				print_number(ip,narg,c);
				putchar(',');
				linux_futexopname((int)*ip);
				ip++;
				narg--;
/*
 *  XXX NOT TESTED
 */
			} else if (ktr->ktr_code == LINUX_SYS_linux_mount) {
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				putchar(',');
				linux_mountflagsname((int)*ip);
				c=',';
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_waitid) {
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				putchar(',');
				linux_waitoptname((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_openat) {
				int flags;
				int mode;
				putchar('(');
				linux_openatdfdname((int)*ip, decimal);
				c=',';
				ip++;
				narg--;
				print_number(ip,narg,c);
				flags = *ip;
				mode = *++ip;
				putchar(',');
				linux_flagsandmodename(flags, mode, decimal);
				ip++;
				narg-=2;
			} else if (ktr->ktr_code == LINUX_SYS_linux_mknodat) {
				putchar('(');
				linux_openatdfdname((int)*ip, decimal);
				c=',';
				ip++;
				narg--;
				print_number(ip,narg,c);
				putchar(',');
				linux_modename((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_fchownat) {
				putchar('(');
				linux_openatdfdname((int)*ip, decimal);
				ip++;
				narg--;
				c=',';
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				putchar(',');
				linux_newatflagsname((int)*ip, decimal);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_futimesat) {
				putchar('(');
				linux_openatdfdname((int)*ip, decimal);
				c=',';
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_fstatat64) {
				putchar('(');
				linux_openatdfdname((int)*ip, decimal);
				c=',';
				ip++;
				narg--;
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				putchar(',');
				linux_newatflagsname((int)*ip, decimal);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_unlinkat) {
				putchar('(');
				linux_openatdfdname((int)*ip, decimal);
				c=',';
				ip++;
				narg--;
				print_number(ip,narg,c);
				putchar(',');
				linux_newatflagsname((int)*ip, decimal);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_renameat) {
				putchar('(');
				linux_openatdfdname((int)*ip, decimal);
				c=',';
				ip++;
				narg--;
				print_number(ip,narg,c);
				putchar(',');
				linux_openatdfdname((int)*ip, decimal);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_linkat) {
				putchar('(');
				linux_openatdfdname((int)*ip, decimal);
				c=',';
				ip++;
				narg--;
				print_number(ip,narg,c);
				putchar(',');
				linux_openatdfdname((int)*ip, decimal);
				putchar(',');
				ip++;
				narg--;
				linux_newatflagsname((int)*ip, decimal);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_symlinkat) {
				print_number(ip,narg,c);
				putchar(',');
				linux_openatdfdname((int)*ip, decimal);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_readlinkat) {
				putchar('(');
				linux_openatdfdname((int)*ip, decimal);
				c=',';
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_mkdirat) {
				putchar('(');
				linux_openatdfdname((int)*ip, decimal);
				c=',';
				ip++;
				narg--;
				print_number(ip,narg,c);
				putchar(',');
				linux_modename((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_getpriority) {
				putchar('(');
				linux_prioname((int)*ip);
				c=',';
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_setitimer ||
				ktr->ktr_code == LINUX_SYS_linux_getitimer) {
				putchar('(');
				linux_itimername((int)*ip);
				ip++;
				narg--;
				c=',';
			} else if (ktr->ktr_code == LINUX_SYS_linux_clone) {
				putchar('(');
				linux_cloneflagsname((int)*ip);
				c=',';
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_flock) {
				print_number(ip,narg,c);
				putchar(',');
				linux_flockname((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_msync) {
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				putchar(',');
				linux_msyncflagsname((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_mlockall) {
				putchar('(');
				linux_mlockallname((int)*ip);
				c=',';
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_sched_setscheduler) {
				/* pid, tid are decimal */
				(void)printf("(%d,", (int)*ip);
				c=',';
				ip++;
				narg--;
				linux_schedpolicyname((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_mremap) {
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				putchar(',');
				linux_mremapflagsname((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_prctl) {
				putchar('(');
				linux_prctloptionname((int)*ip);
				c=',';
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_madvise) {
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				putchar(',');
				linux_madvisebehavname((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_fadvise64 ||
				ktr->ktr_code == LINUX_SYS_linux_fadvise64_64) {
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				putchar(',');
				linux_fadvisename((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_clock_nanosleep) {
				putchar('(');
				linux_clocktidname((int)*ip);
				ip++;
				narg--;
				putchar(',');
				linux_clockflagsname((int)*ip);
				c=',';
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_mbind) {
				print_number(ip,narg,c);
				print_number(ip,narg,c);
				putchar(',');
				linux_mempolicyname((int)*ip);
				ip++;
				narg--;
			} else if (ktr->ktr_code == LINUX_SYS_linux_signal) {
				putchar('(');
				linux_signame((int)*ip);
				ip++;
				narg--;
				putchar(',');
				linux_sighandlername((long)*ip, decimal);
				c=',';
				ip++;
				narg--;
			}
		}
		while (narg) {
			if (decimal)
				printf("%c%ld", c, (long)*ip);
			else
				printf("%c%#lx", c, (long)*ip);
			c = ',';
			ip++;
			narg--;
		}
		putchar(')');
	}
	putchar('\n');
}

void
linux_ktrsysret(struct ktr_sysret *ktr)
{
	register register_t ret = ktr->ktr_retval;
	register int error = ktr->ktr_error;
	register int code = ktr->ktr_code;

	if (code >= nlinux_syscalls || code < 0)
		printf("[%d] ", code);
	else
		printf("%s ", linux_syscallnames[code]);

	if (error == 0) {
		if (ktr->ktr_code == LINUX_SYS_umask && ret > 0) {
			linux_modename(ret);
		} else if (ktr->ktr_code == LINUX_SYS_linux_signal) {
			linux_sighandlername((long)ret, decimal);
		} else {
			if (fancy) {
				printf("%ld", (long)ret);
				if (ret < 0 || ret > 9)
					printf("/%#lx", (long)ret);
			} else {
				if (decimal)
					printf("%ld", (long)ret);
				else
					printf("%#lx", (long)ret);
			}
		}
	} else if (error == ERESTART)
		printf("RESTART");
	else if (error == EJUSTRETURN)
		printf("JUSTRETURN");
	else {
		if (ktr->ktr_error <= ELAST + 1)
			error = abs(bsd_to_linux_errno[ktr->ktr_error]);
		else
			error = 999;
		printf("-1 errno %d", error);
		if (fancy)
			printf(" %s", strerror(ktr->ktr_error));
	}
	putchar('\n');
}

void
linux_ktrsockaddr(struct sockaddr *sa)
{
/*
 TODO: Support additional address families
	#include <netatm/atm.h>
	struct sockaddr_atm	*atm;
	#include <netnatm/natm.h>
	struct sockaddr_natm	*natm;
	#include <netsmb/netbios.h>
	struct sockaddr_nb	*nb;
*/
	char addr[64];

	/*
	 * note: ktrstruct() has already verified that sa points to a
	 * buffer at least sizeof(struct sockaddr) bytes long and exactly
	 * sa->sa_len bytes long.
	 */
	printf("struct sockaddr { ");
	linux_sockfamilyname(sa->sa_family);
	printf(", ");

#define check_sockaddr_len(n)					\
	if (sa_##n->s##n##_len < sizeof(struct sockaddr_##n)) {	\
		printf("invalid");				\
		break;						\
	}

	switch(sa->sa_family) {
	case AF_INET: {
		struct sockaddr_in	*sa_in;

		sa_in = (struct sockaddr_in *)sa;
		check_sockaddr_len(in);
		inet_ntop(AF_INET, &sa_in->sin_addr, addr, sizeof addr);
		printf("%s:%u", addr, ntohs(sa_in->sin_port));
		break;
	}
#ifdef NETATALK
	case AF_APPLETALK: {
		struct sockaddr_at	*sa_at;
		struct netrange		*nr;

		sa_at = (struct sockaddr_at *)sa;
		check_sockaddr_len(at);
		nr = &sa_at->sat_range.r_netrange;
		printf("%d.%d, %d-%d, %d", ntohs(sa_at->sat_addr.s_net),
			sa_at->sat_addr.s_node, ntohs(nr->nr_firstnet),
			ntohs(nr->nr_lastnet), nr->nr_phase);
		break;
	}
#endif
	case AF_INET6: {
		struct sockaddr_in6	*sa_in6;

		sa_in6 = (struct sockaddr_in6 *)sa;
		check_sockaddr_len(in6);
		inet_ntop(AF_INET6, &sa_in6->sin6_addr, addr, sizeof addr);
		printf("[%s]:%u", addr, htons(sa_in6->sin6_port));
		break;
	}
#ifdef IPX
	case AF_IPX: {
		struct sockaddr_ipx	*sa_ipx;

		sa_ipx = (struct sockaddr_ipx *)sa;
		check_sockaddr_len(ipx);
		/* XXX wish we had ipx_ntop */
		printf("%s", ipx_ntoa(sa_ipx->sipx_addr));
		break;
	}
#endif
	case AF_UNIX: {
		struct sockaddr_un *sa_un;

		sa_un = (struct sockaddr_un *)sa;
		check_sockaddr_len(un);
		printf("%.*s", (int)sizeof(sa_un->sun_path), sa_un->sun_path);
		break;
	}
	default:
		printf("unknown address family");
	}
	printf(" }\n");
}

void
linux_ktrstat(struct stat *statp)
{
	char mode[12], timestr[PATH_MAX + 4];
	struct passwd *pwd;
	struct group  *grp;
	struct tm *tm;

	/*
	 * note: ktrstruct() has already verified that statp points to a
	 * buffer exactly sizeof(struct stat) bytes long.
	 */
	printf("struct stat {");
	strmode(statp->st_mode, mode);
	printf("dev=%ju, ino=%ju, mode=%s, nlink=%ju, ",
		(uintmax_t)statp->st_dev, (uintmax_t)statp->st_ino, mode,
		(uintmax_t)statp->st_nlink);
	if (resolv == 0 || (pwd = getpwuid(statp->st_uid)) == NULL)
		printf("uid=%ju, ", (uintmax_t)statp->st_uid);
	else
		printf("uid=\"%s\", ", pwd->pw_name);
	if (resolv == 0 || (grp = getgrgid(statp->st_gid)) == NULL)
		printf("gid=%ju, ", (uintmax_t)statp->st_gid);
	else
		printf("gid=\"%s\", ", grp->gr_name);
	printf("rdev=%ju, ", (uintmax_t)statp->st_rdev);
	printf("atime=");
	if (resolv == 0)
		printf("%ld", (long)statp->st_atimespec.tv_sec);
	else {
		tm = localtime(&statp->st_atimespec.tv_sec);
		(void)strftime(timestr, sizeof(timestr), TIME_FORMAT, tm);
		printf("\"%s\"", timestr);
	}
	if (statp->st_atimespec.tv_nsec != 0)
		printf(".%09ld, ", statp->st_atimespec.tv_nsec);
	else
		printf(", ");
	printf("stime=");
	if (resolv == 0)
		printf("%ld", (long)statp->st_mtimespec.tv_sec);
	else {
		tm = localtime(&statp->st_mtimespec.tv_sec);
		(void)strftime(timestr, sizeof(timestr), TIME_FORMAT, tm);
		printf("\"%s\"", timestr);
	}
	if (statp->st_mtimespec.tv_nsec != 0)
		printf(".%09ld, ", statp->st_mtimespec.tv_nsec);
	else
		printf(", ");
	printf("ctime=");
	if (resolv == 0)
		printf("%ld", (long)statp->st_ctimespec.tv_sec);
	else {
		tm = localtime(&statp->st_ctimespec.tv_sec);
		(void)strftime(timestr, sizeof(timestr), TIME_FORMAT, tm);
		printf("\"%s\"", timestr);
	}
	if (statp->st_ctimespec.tv_nsec != 0)
		printf(".%09ld, ", statp->st_ctimespec.tv_nsec);
	else
		printf(", ");
	printf("birthtime=");
	if (resolv == 0)
		printf("%ld", (long)statp->st_birthtimespec.tv_sec);
	else {
		tm = localtime(&statp->st_birthtimespec.tv_sec);
		strftime(timestr, sizeof(timestr), TIME_FORMAT, tm);
		printf("\"%s\"", timestr);
	}
	if (statp->st_birthtimespec.tv_nsec != 0)
		printf(".%09ld, ", statp->st_birthtimespec.tv_nsec);
	else
		printf(", ");
	printf("size=%jd, blksize=%ju, blocks=%jd, flags=0x%x",
		(uintmax_t)statp->st_size, (uintmax_t)statp->st_blksize,
		(intmax_t)statp->st_blocks, statp->st_flags);
	printf(" }\n");
}

struct linux_socket_args {
	int domain;
	int type;
	int protocol;
};

void
linux_ktrsocket(struct linux_socket_args *args)
{

	printf("socket_args(");
	linux_sockdomainname(args->domain);
	putchar(',');
	linux_socktypename(args->type);
	putchar(',');
	linux_sockipprotoname(args->protocol);
	printf(")\n");
}

struct linux_setsockopt_args {
	int s;
	int level;
	int optname;
	l_uintptr_t optval;
	int optlen;
};

void
linux_ktrsetsockopt(struct linux_setsockopt_args *args)
{

	printf("setsockopt_args(");
	if (args->level == LINUX_SOL_SOCKET)
		printf("SOL_SOCKET,");
	else
		linux_sockipprotoname(args->level);
	linux_sockoptname(args->optname);
	printf(")\n");
}

struct linux_getsockopt_args {
	int s;
	int level;
	int optname;
	l_uintptr_t optval;
	l_uintptr_t optlen;
};

void
linux_ktrgetsockopt(struct linux_getsockopt_args *args)
{

	printf("getsockopt_args(");
	if (args->level == LINUX_SOL_SOCKET)
		printf("SOL_SOCKET,");
	else
		linux_sockipprotoname(args->level);
	linux_sockoptname(args->optname);
	printf(")\n");
}

void
linux_ktrrlimit(struct l_rlimit *limp)
{

	printf("struct l_rlimit { rlim_cur=");
	if (limp->rlim_cur == (l_ulong)LINUX_RLIM_INFINITY)
		printf("INFINITY");
	else
		print_decimal_switch((long)limp->rlim_cur);
	printf(", rlim_max=");
	if (limp->rlim_max == (l_ulong)LINUX_RLIM_INFINITY)
		printf("INFINITY");
	else
		print_decimal_switch((long)limp->rlim_max);
	printf(" }\n");
}

void
linux_ktrsigset(l_sigset_t *lset)
{
	int l, g = 0;

	printf("struct l_sigset_t { ");
	for (l = 1; l <= LINUX_NSIG; l++) {
		if (LINUX_SIGISMEMBER(*lset, l)) {
			if(g++ > 0)
				printf(",");
			if (l < nlinux_signames)
				printf("SIG%s", linux_signames[l]);
			else
				printf("SIG %d", l);
		}
	}
	if (g == 0)
		printf("empty");
	printf(" }\n");
}

void
linux_ktrsigaction(l_sigaction_t *lact)
{

	printf("struct l_sigaction_t { handler=");
	linux_sighandlername((long)lact->lsa_handler, decimal);
	printf(", flags=");
	linux_sigactionflagsname(lact->lsa_flags);
	printf(" }\n");
}

void
linux_ktrstruct(char *buf, size_t buflen)
{
	char *name, *data;
	size_t namelen, datalen;
	int i;

	for (name = buf, namelen = 0;
	     namelen < buflen && name[namelen] != '\0';
	     ++namelen)
		/* nothing */;
	if (namelen == buflen)
		goto invalid;
	if (name[namelen] != '\0')
		goto invalid;
	data = buf + namelen + 1;
	datalen = buflen - namelen - 1;
	if (datalen == 0)
		goto invalid;
	/* sanity check, '.', '_' also */
	for (i = 0; i < namelen; ++i)
		if ((unsigned char)name[i] != 0x5f &&
		    (unsigned char)name[i] != 0x2e &&
		    !isalpha((unsigned char)name[i]))
			goto invalid_verbose;
	if (strcmp(name, "stat") == 0) {
		if (datalen != sizeof(struct stat))
			goto invalid_verbose;
		linux_ktrstat((struct stat *)data);
	} else if (strcmp(name, "l_rlimit") == 0) {
		if (datalen != sizeof(struct l_rlimit))
			goto invalid_verbose;
		linux_ktrrlimit((struct l_rlimit *)data);
	} else if (strcmp(name, "l_sigset_t") == 0) {
		if (datalen != sizeof(l_sigset_t))
			goto invalid_verbose;
		linux_ktrsigset((l_sigset_t *)data);
	} else if (strcmp(name, "l_sigaction_t") == 0) {
		if (datalen != sizeof(l_sigaction_t))
			goto invalid_verbose;
		linux_ktrsigaction((l_sigaction_t *)data);
	} else if (strcmp(name, "sockaddr") == 0) {
		if (datalen < sizeof(struct sockaddr) ||
		    datalen != ((struct sockaddr *)(data))->sa_len)
			goto invalid_verbose;
		linux_ktrsockaddr((struct sockaddr *)data);
	} else if (strcmp(name, "linux_socket_args") == 0) {
		if (datalen != sizeof(struct linux_socket_args))
			goto invalid_verbose;
		linux_ktrsocket((struct linux_socket_args *)data);
	} else if (strcmp(name, "linux_setsockopt_args") == 0) {
		if (datalen != sizeof(struct linux_setsockopt_args))
			goto invalid_verbose;
		linux_ktrsetsockopt((struct linux_setsockopt_args *)data);
	} else if (strcmp(name, "linux_getsockopt_args") == 0) {
		if (datalen != sizeof(struct linux_getsockopt_args))
			goto invalid_verbose;
		linux_ktrgetsockopt((struct linux_getsockopt_args *)data);
	} else {
		printf("unknown structure: %s\n", name);
	}
	return;
invalid:
	printf("invalid record\n");
	return;
invalid_verbose:
	printf("invalid record: %s,  datalen=%ld\n", name, (long)datalen);
}
